#!/usr/bin/env python

import datetime
import json
import optparse
import os
import re
import py_compile
import sys
import tempfile
import urllib
import xbundle
import pkg_resources

try:
    from collections import OrderedDict
except:
    from ordereddict import OrderedDict

from path import path  # needs path.py
from lxml import etree
from plastexit import plastex2xhtml
from course_tests import AnswerBoxUnitTest, CourseUnitTestSet
from abox import split_args_with_quoted_strings

# from logging import Logger

# -----------------------------------------------------------------------------

DEFAULT_CONFIG = {
    'problem_default_attributes': {
        'showanswer': 'closed',
        'rerandomize': 'never',
    }
}

# -----------------------------------------------------------------------------


def date_parse(datestr, retbad=False, verbose=True):
    '''
    Helpful general function for parsing dates.
    Returns datetime object, or None
    '''
    if not datestr:
        return None

    formats = ['%Y-%m-%dT%H:%M:%SZ',    	# 2013-11-13T21:00:00Z
               '%Y-%m-%dT%H:%M:%S.%f',    	# 2012-12-04T13:48:28.427430
               '%Y-%m-%dT%H:%M:%S.%f+00:00',    	# 2013-12-15T18:33:32.378926+00:00
               '%Y-%m-%dT%H:%M:%S',
               '%Y-%m-%dT%H:%M',		# 2013-02-12T19:00
               '%Y-%m-%d %H:%M:%S',		# 2013-05-29 12:51:48
               '%Y-%m-%d',			# 2013-05-29
               '%B %d, %Y',			# February 25, 2013
               '%B %d, %H:%M, %Y', 		# December 12, 22:00, 2012
               '%B %d, %Y, %H:%M', 		# March 25, 2013, 22:00
               '%B %d %Y, %H:%M',		# January 2 2013, 22:00
               '%B %d %Y', 			# March 13 2014
               '%B %d %H:%M, %Y',		# December 24 05:00, 2012
               ]

    for fmt in formats:
        try:
            dt = datetime.datetime.strptime(datestr, fmt)
            return dt
        except Exception:
            continue

    if verbose:
        print "--> Date %s unparsable" % datestr
    if retbad:
        return "Bad"
    return None

# -----------------------------------------------------------------------------


class latex2edx(object):
    '''
    latex2edx works in three stages:

    1. use plastex to convert .tex file to .xhtml, with special edX macros (via py & zpts)
    2. clean up .xhtml file and convert into single .xml file, "xbundle" format
    3. convert xbundle into directory with standard XML format files for edX

    This script can be run from any directory.
    '''

    DescriptorTags = ['course', 'chapter', 'sequential', 'vertical', 'html', 'problem', 'video',
                      'conditional', 'combinedopenended', 'randomize', 'discussion', 'lti']

    def __init__(self,
                 fn,
                 fp=None,
                 extra_filters=None,
                 latex_string=None,
                 add_wrap=False,
                 extra_xml_filters=None,
                 verbose=False,
                 output_fn=None,
                 output_dir='',
                 do_merge=False,
                 imurl='images',
                 do_images=True,
                 update_policy=False,
                 suppress_policy=False,
                 suppress_verticals=False,
                 section_only=False,
                 xml_only=False,
                 units_only=False,
                 popup_flag=False,
                 allow_dirs=False,
                 output_cutset='',
                 ):
        '''
        extra_xml_filters = list of functions acting on XML, applied to XHTML

        output_cutset = `str` : set to filename to store output course unit tests for answer boxes.  These tests can be run using edxcut.
        '''

        if not output_dir:
            output_dir = os.path.abspath('.')
        self.output_dir = path(output_dir)
        imdir = self.output_dir / 'static/images'

        if do_images:  # make directories only if do_images
            if not os.path.exists(self.output_dir):
                os.mkdir(self.output_dir)
            if not os.path.exists(self.output_dir / 'static'):
                os.mkdir(self.output_dir / 'static')
            if not os.path.exists(imdir):
                os.mkdir(imdir)

        self.p2x = plastex2xhtml(fn, fp=fp, extra_filters=extra_filters,
                                 latex_string=latex_string,
                                 add_wrap=add_wrap,
                                 verbose=verbose,
                                 imdir=imdir,
                                 imurl=imurl,
                                 )
        self.p2x.convert()
        self.xhtml = self.p2x.xhtml
        self.do_merge = do_merge
        self.update_policy = update_policy
        self.suppress_policy = suppress_policy
        self.section_only = section_only
        self.suppress_verticals = suppress_verticals
        self.xml_only = xml_only
        self.units_only = units_only
        self.popup_flag = popup_flag
        self.verbose = verbose
        self.the_xml = None
        self.allow_dirs = allow_dirs
        self.output_cutset = output_cutset

        if output_fn is None or not output_fn:
            if fn.endswith('.tex'):
                output_fn = fn[:-4] + '.xbundle'
            else:
                output_fn = fn + '.xbundle'
        self.output_fn = output_fn

        self.fix_filters = [self.fix_xhtml_descriptor_in_p,
                            self.fix_attrib_string,
                            self.add_url_names,
                            self.fix_table,
                            self.fix_table_p,
                            self.fix_latex_minipage_div,
                            self.handle_refs,
                            self.process_edxcite,
                            self.process_askta,
                            self.process_showhide,
                            self.process_edxxml,
                            self.process_dndtex,  # must come before process_include
                            self.process_include,
                            self.process_includepy,
                            self.process_video,
                            self.process_lti,
                            self.process_general_hint_system,
                            self.check_all_python_scripts,
                            self.handle_policy_settings,
                            ]
        if extra_xml_filters:
            self.fix_filters += extra_xml_filters

        if self.output_cutset:
            self.fix_filters.append(self.generate_course_unit_tests)

        self.URLNAMES = []

    def save_xml(self):
        '''
        Save XML file (as .xbundle, normally) to the output_fn
        '''
        open(self.output_fn, 'w').write(etree.tostring(self.xml, pretty_print=True))

    def export_sections_only(self):
        '''
        Export sequentials only (no course, no chapters).
        Also save the initial XML as the xbundle file
        '''
        self.save_xml()
        xb = xbundle.XBundle(force_studio_format=(not self.suppress_verticals), keep_urls=True)
        xb.dir = self.output_dir

        tags = ['sequential', 'problem', 'html', 'video']
        for tag in tags:
            print "    %s: %d" % (tag, len(self.xml.findall('.//%s' % tag)))

        for seq in self.xml.findall('.//sequential'):
            nprob = len(seq.findall('.//problem'))
            nhtml = len(seq.findall('.//html'))
            print "--> exporting sequential %s (%d problem, %d html)" % (seq.get('display_name', '<unknown display_name>'),
                                                                         nprob, nhtml)
            xb.add_descriptors(seq)
            xb.export_xml_to_directory(seq, dowrite=True)

    def export_units_only(self):
        '''
        Export units (problem, html, video) only (no course, no chapters).
        Also save the initial XML as the xbundle file
        '''
        self.save_xml()
        xb = xbundle.XBundle(force_studio_format=(not self.suppress_verticals), keep_urls=True)
        xb.dir = self.output_dir

        tags = ['problem', 'html', 'video']
        for tag in tags:
            print "    %s: %d" % (tag, len(self.xml.findall('.//%s' % tag)))

        for tag in tags:
            for unit in self.xml.findall('.//%s' % tag):
                print "--> exporting %s (%s) url_name=%s" % (unit.get('display_name', '<unknown display_name>'),
                                                             self.get_filename_and_linenum(unit),
                                                             unit.get('url_name', '<unknown>'),
                                                             )
                xb.add_descriptors(unit)
                xb.export_xml_to_directory(unit, dowrite=True)

    @property
    def xml(self):
        '''
        Compute our XML representation by parsing the XHTML from plastex into XML, then running it through
        all the fix_filters.

        Cache result, so we only do the computation once.

        Returns a string (giving the filter-processed XML representation)
        '''
        if self.the_xml is None:
            xml = etree.fromstring(self.xhtml)
            for filter in self.fix_filters:
                filter(xml)
            self.the_xml = xml
        return self.the_xml

    def convert(self):
        '''
        Convert xhtml to xbundle and then xbundle to directory of XML files.
        if self.do_merge then do not overwrite course files; attempt to merge them.
        '''
        if self.section_only and self.xml_only:
            print "Saving XML to file %s" % self.output_fn
            return self.save_xml()

        if self.section_only:
            # if section_only then only export edXsections (sequentials)
            return self.export_sections_only()

        if self.units_only:
            return self.export_units_only()

        self.xhtml2xbundle()
        self.xb.save(self.output_fn)
        print "xbundle generated (%s): " % self.output_fn
        tags = ['chapter', 'sequential', 'problem', 'html', 'video', 'lti']
        for tag in tags:
            print "    %s: %d" % (tag, len(self.xb.course.findall('.//%s' % tag)))
        if self.xml_only:
            print "Saved xbundle XML to file %s" % self.output_fn
            return
        self.xb.export_to_directory(self.output_dir, xml_only=True)
        print "Course exported to %s/" % self.output_dir

        if self.do_merge and self.xb.overwrite_files:
            self.merge_course()

    def merge_course(self):
        print "    merging files %s" % self.xb.overwrite_files
        for fn in self.xb.overwrite_files:
            if str(fn).endswith('course.xml.new'):
                # course.xml shouldn't need merging
                os.unlink(fn)
            else:
                newcourse = etree.parse(open(fn)).getroot()
                oldfn = fn[:-4]
                oldcourse = etree.parse(open(oldfn)).getroot()
                oldchapters = [x.get('url_name') for x in oldcourse]
                newchapters = []
                for chapter in newcourse:
                    if chapter.get('url_name') in oldchapters:
                        continue  # already in old course, skip
                    oldcourse.append(chapter)  # wasn't in old course, move it there
                    newchapters.append(chapter.get('url_name'))
                self.xb.write_xml_file(oldfn, oldcourse, force_overwrite=True)
                os.unlink(fn)
                print "    added new chapters %s" % newchapters

    def xhtml2xbundle(self):
        '''
        Convert XHTML output of PlasTeX to an edX xbundle file.
        Use lxml to parse the XML and extract the desired parts.
        '''
        xml = self.xml
        no_overwrite = ['course'] if self.do_merge else []
        xb = xbundle.XBundle(force_studio_format=(not self.suppress_verticals), keep_urls=True,
                             no_overwrite=no_overwrite)
        xb.KeepTogetherTags = ['sequential', 'vertical', 'conditional']
        course = xml.find('.//course')
        if course is not None:
            xb.set_course(course)
        self.xb = xb
        return xb

    def handle_policy_settings(self, tree):
        '''
        Policy settings are those normally stored in the policies/semester/policy.json
        file.  These include

        - start       : start date
        - end         : end date
        - due         : due date
        - graded      : true/false for graded or not
        - showanswer  : when to allow "show answer"
        - format      : 'grading format' - ie which collection graded components to be part of

        We (optionally) strip these settings from the XML, and (optionally) save them
        in the policy.json file, by updating that file (and not removing other info there).
        '''
        if (not self.suppress_policy) and (not self.update_policy):
            return

        def fixdate(dtin):
            dt = date_parse(dtin)
            if dt is None:
                print "--> Error: bad date '%s' given for policy setting" % dtin
                raise
            return dt.strftime('%Y-%m-%dT%H:%M')

        def makebool(x):
            if 'true' in x.lower():
                return 'true'
            elif 'false' in x.lower():
                return 'false'
            print "--> Warning: in policy settings turning %s in to false" % x
            return 'false'

        policy_settings = {'start': fixdate, 'end': fixdate, 'due': fixdate,
                           'graded': makebool, 'showanswer': None, 'format': None}

        if self.update_policy:
            course = tree.find('.//course')
            semester = course.get('semester', course.get('url_name'))
            policydir = self.output_dir / 'policies' / semester
            if not policydir.exists():
                print "--> Creating directory %s" % policydir
                os.system('mkdir -p "%s"' % policydir)
            policyfile = policydir / 'policy.json'
            if not policyfile.exists():
                print "--> No existing policy.json, creating default"
                policy = OrderedDict()
                policy["course/%s" % semester] = OrderedDict(
                    start="2012-06-02T02:00",
                    end="2012-08-12T00:00",
                )
            else:
                policy = json.load(open(policyfile), object_pairs_hook=OrderedDict)

            def copy_settings(elem, policy):
                key = "%s/%s" % (elem.tag, elem.get('url_name'))
                if key not in policy:
                    policy[key] = OrderedDict()
                for setting, ffun in policy_settings.items():
                    val = elem.get(setting, None)
                    if val is not None:
                        if ffun is None:
                            sval = val
                        else:
                            try:
                                sval = ffun(val)
                            except Exception:
                                msg = "Error processing element %s in %s" % (elem.tag, self.get_filename_and_linenum(elem))
                                raise Exception(msg)
                        policy[key][setting] = sval

            copy_settings(course, policy)		    # do course first

            for chapter in tree.findall('.//chapter'):
                copy_settings(chapter, policy)
                for sequential in chapter.findall('.//sequential'):
                    copy_settings(sequential, policy)

            with open(policyfile, 'w') as fp:
                fp.write(json.dumps(policy, indent=2))

        def suppress_policy_settings(elem):
            for setting in policy_settings:
                if setting in elem.keys():
                    elem.attrib.pop(setting)

        if self.suppress_policy or self.update_policy:
            for chapter in tree.findall('.//chapter'):
                suppress_policy_settings(chapter)
                for sequential in chapter.findall('.//sequential'):
                    suppress_policy_settings(sequential)

    @staticmethod
    def fix_table(tree):
        '''
        Force tables to have table-layout: auto, no borders on table data
        '''
        for table in tree.findall('.//table'):
            table.set('style', 'table-layout:auto')
            for td in table.findall('.//td'):
                newstyle = td.get('style', '')
                if newstyle:
                    newstyle += '; '
                newstyle += 'border:none'
                td.set('style', newstyle)

    @staticmethod
    def fix_table_p(tree):
        '''
        Force "tabular" tables to not have <p> as top-level within <td>.
        Those <p> mess up table spacing.
        '''
        for table in tree.findall('.//table[@class="tabular"]'):
            for td in table.findall('.//td'):
                if not len(td):
                    continue
                tdtop = td[0]
                if tdtop.tag == 'p':
                    for elem in tdtop:
                        tdtop.addprevious(elem)
                td.text = (td.text or '') + tdtop.text
                td.remove(tdtop)

    @staticmethod
    def fix_latex_minipage_div(tree):
        '''
        latex minipages turn into things like <div style="width:216.81pt" class="minipage">...</div>
        but inline math inside does not render properly.  So change div to text.
        '''
        for div in tree.findall('.//div[@class="minipage"]'):
            div.tag = 'text'

    def set_tmploc(self, tree, locstr):
        '''
        Create a tmploc attribute (if none exists) for the elements:
          tocref, toclabel, ref, label, index,
          table class="equation", table class="eqnarray",
          div class="figure"
        so as to later be able to reference these items by addressing their
        location.
        '''
        for elem in tree.xpath('.//tocref|.//toclabel|.//label|'
                               './/table[@class="equation"]|'
                               './/table[@class="eqnarray"]|'
                               './/div[@class="figure"]|'
                               './/ref|.//index'):
            if elem.get('tmploc') is None:
                elem.set('tmploc', locstr)

    def handle_refs(self, tree):
        '''
        Process references to sections of content -- create section numbering and
        reference should be a link that opens in a new tab to the desired component.
        If the --popups option is specified, equations and figure references open a new window.
        '''
        if self.section_only:
            return
        if self.units_only:
            return
        # EVH: Build course map from tree.
        course = tree.find('.//course')
        if course is None:
            return
        cnumber = course.get('number')
        # EVH: Navigate course and set a 'tmploc' attribute with location for desired items
        maplist = []  # ['loc. str.']
        mapdict = {}  # {'location str.':['URL','display_name','refnum']}
        chapnum = 0
        chapref = seqref = vertref = '0'
        for chapter in tree.findall('.//chapter'):
            chapnum += 1
            if chapter.get('refnum') is not None:
                chapref = chapter.get('refnum')
                seqref = vertref = '0'
            chapurl = chapter.get('url_name')
            locstr = '{}'.format(chapnum)
            maplist.append(locstr)
            mapdict[locstr] = [
                '{}'.format(chapurl),
                chapter.get('display_name'), chapref]
            labels = [
                chapter.find('./p/label'), chapter.find('./label'),
                chapter.find('./p/toclabel'), chapter.find('./toclabel')]
            for label in labels:
                if label is not None:
                    label.set('tmploc', locstr + '.0')
            seqnum = 0
            for child1 in chapter:
                if child1.tag == 'p' and (child1.find('./') is not None):
                    seq = child1[0]
                else:
                    seq = child1
                if seq.tag not in ['sequential', 'vertical', 'section']:
                    continue
                seqnum += 1
                if seq.get('refnum') is not None:
                    seqref = seq.get('refnum')
                    vertref = '0'
                sequrl = seq.get('url_name')
                locstr = '{}.{}'.format(chapnum, seqnum)
                maplist.append(locstr)
                mapdict[locstr] = [
                    '{}/{}'.format(chapurl, sequrl),
                    seq.get('display_name'), '.'.join([chapref, seqref])]
                labels = [
                    seq.find('./p/label'), seq.find('./label'),
                    seq.find('./p/toclabel'), seq.find('./toclabel')]
                for label in labels:
                    if label is not None:
                        label.set('tmploc', locstr + '.0')
                if seqnum == 1:
                    mapdict['{}'.format(chapnum)][0] = (
                        '{}/{}/1'.format(chapurl, sequrl))
                vertnum = 0
                for child2 in seq:
                    if child2.tag == 'p' and (child2.find('./') is not None):
                        vert = child2.find('./')
                    else:
                        vert = child2
                    if vert.tag not in ['sequential', 'vertical', 'section',
                                        'problem', 'html']:
                        continue
                    vertnum += 1
                    if vert.get('refnum') is not None:
                        vertref = vert.get('refnum')
                    locstr = '{}.{}.{}'.format(chapnum, seqnum, vertnum)
                    maplist.append(locstr)
                    mapdict[locstr] = [
                        '{}/{}/{}'.format(chapurl, sequrl, vertnum),
                        vert.get('display_name'),
                        '.'.join([chapref, seqref, vertref])]
                    labels = [
                        vert.find('./p/label'), vert.find('./label'),
                        vert.find('./p/toclabel'), vert.find('./toclabel')]
                    for label in labels:
                        if label is not None:
                            label.set('tmploc', locstr + '.0')
                    self.set_tmploc(vert, locstr)
                locstr = '.'.join(locstr.split('.')[:-1])
                self.set_tmploc(seq, locstr)
            locstr = '.'.join(locstr.split('.')[:-1])
            self.set_tmploc(chapter, locstr)
        # EVH 01-13-15: tmploc assignment added at course level
        self.set_tmploc(course, '0')
        mapdict['0'] = ['#', cnumber, '0']
        # EVH: Handle figure references. Search for labels and build dictionary
        figdict = {}  # {'figlabel':'fignum'}
        figattrib = {}  # {'figlabel':{'attrib':'value'}}
        for fig in tree.findall('.//div[@class="figure"]'):
            locstr = fig.attrib.pop('tmploc')
            # Retrieve Figure number if it is captioned
            caption = fig.find('.//div[@class="caption"]/b')
            if caption is not None:
                fignum = caption.text.split(' ')[1]
            figlabel = None
            label = fig.find('.//label')
            if label is not None:
                figlabel = label.text
                figdict[figlabel] = fignum
                plabel = label.getparent()
                if plabel.tag == 'p':  # TODO: Find a clean way to build eTree
                    label = plabel
                    plabel = plabel.getparent()
                plabel.remove(label)
            if figlabel is not None:
                # CHAD: for multi-image figures, collect all the image names
                # TODO: Find example and investigate how to refine (as above)
                fig.set('id', 'fig{}'.format(fignum))
                figattrib[figlabel] = {
                    'href': '{}#fig{}'.format(mapdict[locstr][0], fignum),
                    'onClick': 'location.reload()'}
                if self.popup_flag:
                    imgsrcs = []
                    for img in fig.findall('.//img'):
                        imgsrc = img.get('src')
                        imgsrcs.append(imgsrc)
                    if len(imgsrcs) == 1:  # single image figure
                        figfile = imgsrcs[0]
                        # TODO: Find a way to resize popup window to the figure
                        figattrib[figlabel] = {
                            'href': '{}'.format(figfile),
                            'onClick': ("window.open(this.href, \'{}\',"
                                        "\'width=400,height=200\',"
                                        "\'toolbar=1\'); return false;".
                                        format(cnumber))}
                    else:  # multi-image figure
                        htmlbodycontent = ""
                        for figfile in imgsrcs:
                            htmlbodycontent += (
                                "<img src=\"{}\" width=\"400\""
                                "height=\"200\">".format(figfile))
                        htmlstr = (
                            "\'<html><head></head><body>{}</body></html>\'".
                            format(htmlbodycontent))
                        figattrib[figlabel] = {
                            'onClick': ("return newWindow({}, 'Figure {}');".
                                        format(htmlstr, fignum)),
                            'href': 'javascript: void(0)'}
        # EVH: Build cross reference dictionaries for ToC refs
        toclist = []  # ['toclabel']
        tocdict = {}  # {'toclabel',['locstr','label text']}
        labeldict = {}  # {'labeltag':['loc. URL','chapnum.labelnum']}
        tocrefdict = {}  # {'tocref':[['locstr'],['parent name']]}
        labelcnt = {}  # {'labeltag':cnt}
        chapref = '0'
        for label in tree.xpath('.//label|//toclabel'):
            locstr = label.get('tmploc')
            if locstr.split('.')[-1] == '0':
                locstr = locstr[:-2]
                hlabel = True
            else:
                hlabel = False
            locref = mapdict[locstr][2]
            labelref = label.text
            if locref.split('.')[0] != chapref:
                chapref = locref.split('.')[0]
                labelcnt = {}  # Reset label count
            if hlabel:
                labeldict[labelref] = [mapdict[locstr][0], locref]
            else:
                labeltag = labelref.split(':')[0]
                if labeltag in labelcnt:
                    labelcnt[labeltag] += 1
                else:
                    labelcnt[labeltag] = 1
                if chapref == '0':
                    labelnum = '{}'.format(labelcnt[labeltag])
                else:
                    labelnum = '{}.{}'.format(chapref, labelcnt[labeltag])
                if ':' in labelref:
                    labeltag += ':' + labelnum
                labeldict[labelref] = [mapdict[locstr][0], labeltag]
            # Get label tail and parent text, and remove label
            labeltail = label.tail
            plabel = label.getparent()
            ptext = plabel.text
            if labeltail != ' ' and (labeltail is not None):
                if ptext == '\n' or (ptext is None):
                    ptext = labeltail
                else:
                    ptext = ptext[:-1] + labeltail  # remove ptext CR, add tail
            if label.tag == 'toclabel':
                toclist.append(labelref)
                tocdict[labelref] = [locstr, ptext]
                # Change URL to point to the ToC location
                labeldict[labelref][0] = ('../tocindex/#anchor{}'.
                                          format(labeldict[labelref][1].
                                                 upper().replace(r'.', 'p').
                                                 replace(':', '')))
            if plabel.tag == 'p':
                label = plabel
                plabel = plabel.getparent()
            plabel.text = ptext
            plabel.remove(label)
        for tocref in tree.findall('.//tocref'):
            tagref = tocref.text
            locstr = tocref.get('tmploc')
            paref = tocref.getparent()
            paref.text += tocref.tail
            paref.remove(tocref)
            while paref.tag not in ['html', 'problem', 'vertical']:
                paref = paref.getparent()
                pareftag = paref.tag
            # EVH: Prepend letter to identify content type
            if pareftag == 'vertical':
                parind = []
                for i, child in enumerate(paref):
                    if child.tag in ['html', 'problem']:
                        parind = min(parind, i)
                    if child.tag == 'problem':
                        pareftag = 'problem'
                paref = paref[parind]
            if pareftag == 'problem':
                parefname = 'P' + paref.get('display_name')
                oldtag = paref.get('measureable_outcomes')
                tagname = tagref
                if ':' in tagname:
                    tagname = tagname.split(':')[1]
                if oldtag is None:
                    newtag = tagname
                else:
                    newtag = oldtag + ',' + tagname
                paref.set('measureable_outcomes', newtag)
            else:
                parefname = 'H' + paref.get('display_name')
            if tagref in tocrefdict:
                tocrefdict[tagref][0].append(locstr)
                tocrefdict[tagref][1].append(parefname)
            else:
                tocrefdict[tagref] = [[locstr], [parefname]]
            taglist = paref.find(".//p[@id='taglist']")
            if taglist is None:
                taglist = etree.Element('p', id='taglist', tmploc=locstr,
                                        tags=tagref)
                paref.insert(0, taglist)
            else:
                taglist.set('tags', taglist.get('tags') + ',' + tagref)
        # EVH: Parse taglist to create ToC button links at the top of each vert
        for taglist in tree.findall(".//p[@id='taglist']"):
            locstr = taglist.get('tmploc')
            tags = taglist.get('tags').split(',')
            for tocref in tags:
                tocrefid = tocref
                if ':' in tocrefid:
                    tocrefid = tocrefid.split(':')[1]
                if tocref not in labeldict:
                    continue
                link = etree.SubElement(
                    taglist, 'button',
                    {'type': "button", 'border-radius': "2px",
                     'title': "{}:\n{}".format(labeldict[tocref][1].upper().
                                               replace(':', ''),
                                               tocdict[tocref][1]),
                     'style': "cursor:pointer", 'class': "mo_button",
                     'onClick': ("window.location.href='{}{}'".
                                 format('../' * (len(locstr.split('.')) - 1),
                                        labeldict[tocref][0]))})
                link.text = labeldict[tocref][1].upper().replace(':', '')
                link.set('id', tocrefid)
        tochead = ['h2', 'h3', 'h4']
        if len(toclist) != 0:
            # EVH: Start building tocindex.html
            toctree = etree.Element('html')
            toctree.append(etree.fromstring('<head></head>'))
            tocbody = etree.SubElement(toctree, 'body')
            tocbody.append(etree.Element('h1'))
            tocbody[0].text = 'Table of Contents'
        while len(toclist) != 0:
            hlabel = False
            toclabel = toclist.pop(0)
            tocloc = tocdict[toclabel][0]
            tocname = tocdict[toclabel][1]
            if tocloc.split('.')[-1] == '0':
                hlabel = True
                tocloc = tocloc[:-2]
            while tocloc in maplist:
                tocentry = maplist.pop(0)
                entryname = mapdict[tocentry][1]
                toclevel = len(tocentry.split('.'))
                if toclevel == 1:
                    if tocentry.split('.')[0] != '1':
                        tocbody.append(etree.Element('br'))
                    # Insert chapter titles if no toclabel exist
                    if not hlabel:
                        tocitem = etree.Element(
                            'a', {'href': ('../courseware/' +
                                           mapdict[tocentry][0])})
                        tocitem.append(etree.Element('h2'))
                        tocitem[0].text = entryname
                        tocbody.append(tocitem)
            if toclabel in tocrefdict:
                toctag = labeldict[toclabel][1].replace(':', '')
                tocbody.append(etree.Element(
                    'a', {'name': 'anchor{}'.format(toctag.upper().
                                                    replace('.', 'p'))}))
                toctable = etree.Element(
                    'table',
                    {'id': 'label',
                     'class': 'wikitable collapsible collapsed'})
                toctable.append(etree.Element('tbody'))
                tablecont = etree.SubElement(toctable[0], 'tr')
                tablecont = etree.SubElement(tablecont, 'th')
                tablecont.append(etree.Element(
                    'a',
                    {'id': 'ind{}l'.format(toctag.replace('.', 'p')),
                     'onclick': ("$('#ind{}').toggle();return false;".
                                 format(toctag.replace('.', 'p'))),
                     'name': 'ind{}l'.format(toctag.replace('.', 'p')),
                     'href': '#'}))
                if hlabel:
                    tablecont = etree.SubElement(
                        tablecont[0], tochead[toclevel - 1])
                    tablecont.text = entryname
                else:
                    tablecont[0].append(etree.Element(
                        'strong', {'itemprop': 'name'}))
                    tablecont[0][0].text = toctag.upper()
                    tablecont = etree.SubElement(
                        tablecont, 'span', {'itemprop': 'description'})

                    tablecont.text = tocname

                tablecont = etree.SubElement(
                    toctable[0], 'tr',
                    {'id': 'ind{}'.format(toctag.replace('.', 'p')),
                     'style': 'display:none'})
                tablecont = etree.SubElement(tablecont, 'td')
                tablecont.append(etree.Element('h4'))
                tablecont[0].text = 'Learn'
                tablecont.append(etree.Element(
                    'ul', {'class': '{}learn'.format(toclabel.split(':')[0].
                                                     upper())}))
                tablecont.append(etree.Element('h4'))
                tablecont[2].text = 'Assess'
                tablecont.append(etree.Element(
                    'ul', {'class': '{}assess'.format(toclabel.split(':')[0].
                                                      upper())}))
                tocrefs = tocrefdict.pop(toclabel)
                tocrefnames = tocrefs[1]
                tocrefs = tocrefs[0]
                for tocref in tocrefs:
                    tableli = etree.Element('li')
                    tableli.append(etree.Element(
                        'a', {'href': ('../courseware/' +
                                       mapdict[tocref][0]),
                              'itemprop': 'name'}))
                    tocrefname = tocrefnames.pop(0)
                    tableli[0].text = tocrefname[1:]
                    if tocrefname[0] == 'H':
                        tablecont[1].append(tableli)
                    else:
                        tablecont[3].append(tableli)
            else:
                toctable = etree.Element('a', {'href': ('../courseware/' +
                                                        mapdict[tocloc][0])})
                if hlabel:
                    tablecont = etree.SubElement(
                        toctable, tochead[toclevel - 1])
                    tablecont.text = entryname
                else:
                    toctable.append(etree.Element(
                        'strong', {'itemprop': 'name'}))
                    toctable[0].text = (labeldict[toclabel][1].upper().
                                        replace(':', ''))
                    tablecont = etree.SubElement(
                        toctable, 'span', {'itemprop': 'description'})
                    tablecont.text = tocname
            tocbody.append(toctable)
        if len(tocdict) != 0:
            print "Writing ToC index content..."
            if not os.path.exists(self.output_dir):
                os.mkdir(self.output_dir)
            if not os.path.exists(self.output_dir / 'tabs'):
                os.mkdir(self.output_dir / 'tabs')
            tocf = open(self.output_dir / 'tabs' / 'tocindex.html', 'w')
            tocf.write(etree.tostring(
                toctree, method='html', pretty_print=True))
            tocf.close()

        class MissingLabel(Exception):
            '''
            Exception raised when a referrence to a non-existent label is found
            '''

            def __init__(self, value):
                '''
                Add a new value to the Exception call
                Arg: value (str)
                '''
                self.value = value

            def __str__(self):
                '''
                Return the value of the Exception as a string
                '''
                return repr(self.value)

        # EVH: Check for unused tocrefs
        for tocref in tocrefdict:
            try:
                raise MissingLabel(tocref)
            except MissingLabel as referr:
                print ('WARNING: There is a reference to non-existent '
                       'ToC label: {}'.format(str(referr)))

        # EVH: Handle equation refs. Search for labels and build dictionaries
        eqndict = {}  # {'eqnlabel':'eqnnum'}
        eqnattrib = {}  # {'eqnlabel':{'attrib':'value'}}
        chapref = '0'
        eqncnt = 0
        for table in tree.xpath('.//table[@class="equation"]|'
                                './/table[@class="eqnarray"]'):
            if 'tmploc' in table.attrib:
                locstr = table.attrib.pop('tmploc')
            if not locstr:
                continue
            locref = mapdict[locstr][2]
            if chapref != locref.split('.')[0]:
                chapref = locref.split('.')[0]
                eqncnt = 0
            for tr in table.findall('.//tr'):  # Max one label per table row
                eqnnumcell = None
                eqnlabel = []
                for td in tr.findall('.//td'):
                    if td.get('class') == 'eqnnum' and td.text != u'\u00A0':
                        eqnnumcell = td
                        # EVH: Use plasTeX output to handle equation numbering
                        eqncnt += 1
                        if chapref == '0':
                            eqnnum = '{}'.format(eqncnt)
                        else:
                            eqnnum = '{}.{}'.format(chapref, eqncnt)
                        tr.remove(eqnnumcell)
                        eqnnumcell = etree.SubElement(
                            tr, "td", attrib=eqnnumcell.attrib)
                        eqnnumcell.text = '({})'.format(eqnnum)
                        eqnnumsty = eqnnumcell.get('style')
                        eqnnumsty = re.sub('text-align:[a-zA-Z]+;', '', eqnnumsty)
                        eqnnumsty += ';text-align:right'
                        eqnnumcell.set('style', eqnnumsty)
                    elif td.text is not None:
                        eqncontent = td.text
                        if re.search(r'\\label\{(.*?)\}',
                                     eqncontent, re.S) is not None:
                            eqnlabel = re.findall(r'\\label\{(.*?)\}',
                                                  eqncontent, re.S)
                            eqncontent = re.sub(r'\\label{.*?}', r'',
                                                eqncontent)
                            td.text = eqncontent
                if len(eqnlabel) != 0:
                    # NOTE: EVH 2015-07-24 find a better way to handle labels
                    # to unnumbered equations
                    if eqnnumcell is None:
                        eqnnum = 'NaN'
                    eqnlabel = eqnlabel[0]
                    eqnlabel = eqnlabel.replace(' ', '')
                    eqndict[eqnlabel] = '{}'.format(eqnnum)
                    # EVH: Set id for linking if pop-up flag is False
                    tr.set('id', 'eqn{}'.format(eqnnum.replace('.', 'p')))
                    eqnattrib[eqnlabel] = {
                        'href': '{}#eqn{}'.format(mapdict[locstr][0],
                                                  eqnnum.replace('.', 'p')),
                        'onClick': 'location.reload()'}
                if self.popup_flag and len(eqnlabel) != 0:
                    eqnattrib[eqnlabel]['href'] = 'javascript: void(0)'
                    eqntablecontent = (etree.tostring(
                        tr, encoding="utf-8", method="html")).rstrip()
                    eqntablecontent = ''.join(re.findall(
                        r'\[mathjax[a-z]*\](.*?)\[/mathjax[a-z]*\]',
                        eqntablecontent, re.S))
                    eqntablecontent = re.escape('$$' + eqntablecontent + '$$')
                    if re.search(r'\\boxed', eqntablecontent,
                                 re.S) is not None:
                        eqntablecontent = eqntablecontent.replace(
                            r'\boxed', '')
                    eqntablecontent = (
                        "<table width=\"100%%\" cellspacing=\"0\""
                        "cellpadding=\"7\" style=\"table-layout:auto;"
                        "border-style:hidden\"><tr><td style=\"width:80%%;"
                        "vertical-align:middle;text-align:center;"
                        "border-style:hidden\">{}</td><td style=\"width:20%%;"
                        "vertical-align:middle;text-align:left;"
                        "border-style:hidden\">({})</td></tr></table>".
                        format(eqntablecontent, eqnnum))
                    mathjax = (
                        "<script type=\"text/javascript\" src=\"https://edx-"
                        "static.s3.amazonaws.com/mathjax-MathJax-727332c/Math"
                        "Jax.js?config=TeX-MML-AM_HTMLorMML-full\"> </script>")
                    htmlstr = (
                        "\'<html><head>{}</head><body>{}</body></html>\'".
                        format(mathjax, eqntablecontent))
                    eqnattrib[eqnlabel]['onClick'] = (
                        "return newWindow({}, \'Equation {}\');".
                        format(htmlstr, eqnnum))

        # EVH: Build keymap dictionary for keywords specified by the \index
        # command
        keymap = {}  # {keyword: [[URL], [display_name]]}
        for indexref in tree.findall('.//index'):
            locstr = indexref.get('tmploc')
            keyref = indexref.text
            if keyref in keymap:
                keymap[keyref][0].append(mapdict[locstr][0])
                keymap[keyref][1].append(mapdict[locstr][1])
            else:
                keymap[keyref] = [[mapdict[locstr][0]],
                                  [mapdict[locstr][1]]]
            p = indexref.getparent()
            p.remove(indexref)

        # EVH: Find and replace references everywhere with ref number and link
        for aref in tree.findall('.//ref'):
            reflabel = aref.text
            locstr = aref.attrib.pop('tmploc')
            if self.popup_flag:
                relurl = ''
            else:
                relurl = '../' * (len(locstr.split('.')) - 1)
            if reflabel in figdict:
                aref.tag = 'a'
                aref.text = figdict[reflabel]
                for attrib in figattrib[reflabel]:
                    aref.set(attrib, figattrib[reflabel][attrib])
                rawref = aref.get('href')
                aref.set('href', (relurl + rawref))
            elif reflabel in labeldict:
                aref.tag = 'a'
                aref.text = labeldict[reflabel][1].replace(':', ' ')
                aref.set('href', ('../' * (len(locstr.split('.')) - 1) +
                                  labeldict[reflabel][0]))
                aref.set('target', "_blank")
            elif reflabel in eqndict:
                aref.tag = 'a'
                aref.text = eqndict[reflabel]
                for attrib in eqnattrib[reflabel]:
                    aref.set(attrib, eqnattrib[reflabel][attrib])
                rawref = aref.get('href')
                aref.set('href', (relurl + rawref))
            else:
                try:
                    raise MissingLabel(aref.text)
                except MissingLabel as referr:
                    print ('WARNING: There is a reference to non-existent '
                           'label: {}'.format(str(referr)))

        if len(keymap) != 0:
            if not os.path.exists(self.output_dir):
                os.mkdir(self.output_dir)
            if not os.path.exists(self.output_dir / 'static'):
                os.mkdir(self.output_dir / 'static')
            print "Writing key_map.json to static/ ..."
            kwjson = open(self.output_dir / 'static' / 'key_map.json', 'w')
            kwjson.write(json.dumps(keymap, default=lambda o: o.__dict__))
            kwjson.close()

    def process_askta(self, tree):
        '''
        add "Ask TA!" links
        arguments are taken as space delimited settings

        if "settings" set, then:
           - save key,value for next uses of edXaskta
           - do not display a link

        examples:

        % sets settings, does not display link
        \edXaskta{settings=1 label="Ask TA!" url_base="htps://edx.org/mycourse" to:"me@example.edu" cc:"ta@example.edu"}

        % displays Email TA link
        \edXaskta{label="Email TA" subject:"help"}
        '''

        special_attribs = ['url_base', 'cnt', 'label']

        if not hasattr(self, 'askta_data'):
            subject = "Question about {name}"
            body = "This is a question about the problem at COURSE_URL/{url_name}\n\n"
            self.askta_data = {'cnt': 0, 'label': 'Ask TA!', 'to': '', 'cc': '', 'subject': subject,
                               'body': body,
                               'url_base': 'https://edx.org',
                               }

        for askta in tree.findall('.//askta'):
            text = askta.text
            args = {}
            if text:
                argset = split_args_with_quoted_strings(text)
                try:
                    args = dict([x.split('=', 1) for x in argset])
                    for arg in args:
                        args[arg] = self.stripquotes(args[arg], checkinternal=True)
                except Exception, err:
                    print "Error %s" % err
                    print "Failed in parsing args to edXaskta = %s" % text
                    raise
                if 'settings' in args:
                    args.pop('settings')
                    self.askta_data.update(args)
                    print "askTA settings updated: %s" % self.askta_data
                    # remove this element from xml tree
                    # self.remove_parent_p(askta)
                    p = askta.getparent()
                    p.remove(askta)
                    continue

            # generate button link, something like this:
            #   <input style="float:right" class="check Check" type="button" value="Ask TA!" onclick="SendMail();"/>
            # <script type="text/javascript">
            # var amp = String.fromCharCode(38);
            # function SendMail() {
            #          var link = "mailto:me@example.com"
            #             + "?cc=myCCaddress@example.com"
            #             + amp + "subject=" + escape("This is my subject")
            #             + amp + "body=";
            #          window.open(link,'AskTA', "height=500,width=700");
            # }
            # </script>

            data = {}
            data.update(self.askta_data)
            data.update(args)

            display_name = ''
            url_name = ''
            for parent in askta.xpath('ancestor::*')[::-1]:
                display_name = parent.get('display_name', '')
                if display_name:
                    url_name = parent.get('url_name')
                    break

            data['subject'] = data['subject'].format(name=display_name)
            data['body'] = data['body'].format(url_name=url_name, **data)

            self.askta_data['cnt'] += 1
            smfn = 'SendMail_%d' % self.askta_data['cnt']

            askta.tag = 'span'
            askta.text = ''

            atin = etree.SubElement(askta, 'input')
            atin.set('style', 'float:right')
            atin.set('class', 'check Check')
            atin.set('value', data['label'])
            atin.set('type', 'button')
            atin.set('onclick', '%s();' % smfn)

            for attrib in special_attribs:
                data.pop(attrib)

            atlid = 'aturl_%s' % self.askta_data['cnt']
            atlink = etree.SubElement(askta, 'a')
            atlink.set('style', 'display:none')
            atlink.set('href', '/course/jump_to_id')
            atlink.set('id', atlid)

            mailto = 'mailto:%s' % data['to']
            data.pop('to')
            body = data.pop('body')
            mailto += '?' + urllib.urlencode(data)
            mailto += '&' + urllib.urlencode({'body': body})

            jscode = ('\nfunction %s() {\n'
                      '    var cu = encodeURI(window.location.origin + $("#%s").attr("href"));\n'
                      '    var link = "%s";\n'
                      '    link = link.replace("COURSE_URL", cu);\n'
                      '    link = link.replace(/&/g, String.fromCharCode(38));\n'
                      '    console.log(link);\n'
                      '    Logger.log("askta",{link:link});\n'
                      '    window.open(link, "AskTA", "height=500,width=700"); \n'
                      '}') % (smfn, atlid, mailto)

            script = etree.SubElement(askta, 'script')
            script.set('type', 'text/javascript')
            script.text = jscode

    @staticmethod
    def stripquotes(x, checkinternal=False):
        if x.startswith('"') and x.endswith('"'):
            if checkinternal and '"' in x[1:-1]:
                return x
            return x[1:-1]
        if x.startswith("'") and x.endswith("'"):
            return x[1:-1]
        return x

    def process_edxcite(self, tree):
        '''
        Add citation link visible on mouse hoover.
        '''
        if not hasattr(self, 'edxcitenum'):
            self.edxcitenum = 0
        for edxcite in tree.findall('.//edxcite'):
            self.edxcitenum += 1
            ref = edxcite.get('ref', None)
            if ref is None or not ref:
                ref = '[%d]' % self.edxcitenum
            text = edxcite.text
            exc = etree.Element('a')
            edxcite.addnext(exc)
            sup = etree.SubElement(exc, 'sup')
            sup.text = ref
            exc.set('href', '#')
            exc.set('title', text)
            # print "  --> %s" % etree.tostring(exc)
            p = edxcite.getparent()
            p.remove(edxcite)

    @staticmethod
    def remove_parent_p(xml):
        '''
        If xml is inside an otherwise empty <p>, then push it up and remove the <p>
        '''
        p = xml.getparent()
        todrop = xml
        where2add = xml
        if p.tag == 'p' and not p.text.strip() and len(p) == 1:	 # if in empty <p> then remove that <p>
            todrop = p
            where2add = p
            p = p.getparent()

        # move from xml to parent: text, children, and tail
        if xml.text:
            if xml.getprevious() is not None:
                if xml.getprevious().tail:
                    xml.getprevious().tail += xml.text
                else:
                    xml.getprevious().tail = xml.text
            else:
                if p.text:
                    p.text += xml.text
                else:
                    p.text = xml.text
        for child in xml:
            where2add.addprevious(child)
        if xml.tail:
            if 'child' in locals():
                if child.tail:
                    child.tail += xml.tail
                else:
                    child.tail = xml.tail
            else:
                if p.text:
                    p.text += xml.tail
                else:
                    p.text = xml.tail
        p.remove(todrop)

    def process_edxxml(self, tree):
        '''
        move content of edXxml into body
        If edXxml is within a <p> then drop the <p>.  This allows edXxml to be used for discussion and video.
        '''
        for edxxml in tree.findall('.//edxxml'):
            self.remove_parent_p(edxxml)

    def process_video(self, tree):
        '''
        If the "youtubeid" begins with "http" then make the video an html5 video.
        '''
        for video in tree.findall('.//video'):
            ytid = video.get('youtube_id_1_0')
            if ytid.startswith('http'):
                video.set('html5_sources', '["%s"]' % ytid)
                video.set('youtube_id_1_0', '')
                vsource = etree.Element('source')
                vsource.set('src', ytid)
                video.append(vsource)

    def process_lti(self, tree):
        '''
        For LTI elements, any custom_* attributes should be moved into a special single
        "custom_parameters" attribute.
        '''
        for lti in tree.findall('.//lti'):
            cplist = []
            for key, val in lti.attrib.items():
                if key.startswith('custom_'):
                    cplist.append("%s=%s" % (key[7:], val))  # strip "custom_" prefix
                    lti.attrib.pop(key)
            if cplist:
                lti.set('custom_parameters', '[%s]' % ', '.join(['"' + x + '"' for x in cplist]))
            if self.verbose:
                print "    lti %s, cp=%s" % (lti, lti.get('custom_parameters'))

    def process_showhide(self, tree):
        for showhide in tree.findall('.//edxshowhide'):
            desc = showhide.get('description', '')
            oneup = showhide.getparent()
            newsh = etree.SubElement(oneup, 'div', {'class': 'hideshowbox'})
            sub1 = etree.SubElement(newsh, 'h4',
                                    {'onclick': 'hideshow(this);',
                                     'style': 'margin: 0px'})
            sub1.text = desc
            etree.SubElement(sub1, 'span',
                             {'class': 'icon-caret-down toggleimage'})
            newsh.append(showhide)
            showhide.tag = 'div'  # change edxshowhide tag
            showhide.attrib.pop('description')  # remove description
            showhide.set('class', 'hideshowcontent')
            sub2 = etree.SubElement(newsh, 'p',
                                    {'class': 'hideshowbottom',
                                     'onclick': 'hideshow(this);',
                                     'style': 'margin: 0px'})
            subsub2 = etree.SubElement(sub2, 'a',
                                       {'href': 'javascript: {return false;}'})
            subsub2.text = 'Show'
            par = newsh.getparent()
            while (par.tag != 'html') and (par.tag != 'problem'):
                par = par.getparent()
                if par.tag == 'vertical' or par.tag == 'sequential':
                    raise Exception("Must use showhide inside html or "
                                    "problem element")
                    break
            scriptforsh = etree.Element('SCRIPT',
                                        {'type': 'text/javascript',
                                         'src': '/static/latex2edx.js'})
            styleforsh = etree.Element('LINK',
                                       {'type': 'text/css',
                                        'rel': 'stylesheet',
                                        'href': '/static/latex2edx.css'})
            if len(par.findall('.//SCRIPT[@src="/static/latex2edx.js"]')) == 0:
                par.append(scriptforsh)
                par.append(styleforsh)
                staticdir = self.output_dir / 'static'
                if not os.path.exists(staticdir):
                    if not os.path.exists(self.output_dir):
                        os.mkdir(self.output_dir)
                    os.mkdir(staticdir)
                if not os.path.exists(staticdir / 'latex2edx.js'):
                    l2ejs = pkg_resources.resource_filename(__name__,
                                                            'latex2edx.js')
                    cmd = 'cp {} {}/'.format(l2ejs, staticdir)
                    print '----> Copying showhide JavaScript: {}'.format(cmd)
                    sys.stdout.flush()
                    os.system(cmd)
                if not os.path.exists(staticdir / 'latex2edx.css'):
                    l2ecss = pkg_resources.resource_filename(__name__,
                                                             'latex2edx.css')
                    cmd = 'cp {} {}/'.format(l2ecss, staticdir)
                    print '----> Copyting showhide CSS: {}'.format(cmd)
                    sys.stdout.flush()
                    os.system(cmd)

    def process_include(self, tree, do_python=False):
        '''
        Include XML or python file.

        For python files, wrap inside <script><![CDATA[ ... ]]></script>
        '''
        tag = './/edxinclude'
        cmd = 'edXinclude'
        if do_python:
            tag += 'py'
            cmd += "py"
        for include in tree.findall(tag):
            incfn = include.text
            linenum = include.get('linenum', '<unavailable>')
            texfn = include.get('filename', '<unavailable>')
            if incfn is None:
                print "Error: %s must specify file to include!" % cmd
                raise Exception(self.standard_error_msg(include))
            incfn = incfn.strip()
            if not os.path.exists(incfn):
                print "Error: include file %s does not exist!" % incfn
                raise Exception(self.standard_error_msg(include))
            try:
                incdata = open(incfn).read()
            except Exception, err:
                print "Error %s: cannot open include file %s to read" % (err, incfn)
                raise Exception(self.standard_error_msg(include))

            # if python script, then check its syntax
            if do_python:
                try:
                    py_compile.compile(incfn, doraise=True)
                except Exception as err:
                    print "Error in python script %s! Err=%s" % (incfn, err)
                    print "Aborting!"
                    raise Exception(self.standard_error_msg(include))

            try:
                if do_python:
                    incxml = etree.fromstring('<script><![CDATA[\n%s\n]]></script>' % incdata)
                else:
                    incxml = etree.fromstring(incdata)
            except Exception, err:
                print "Error %s parsing XML for include file %s" % (err, incfn)
                print "See tex file %s line %s" % (texfn, linenum)
                raise Exception(self.standard_error_msg(include))

        # remove parent <p> if it exists
            parent = include.getparent()
            pp = parent.getparent()
            if parent.tag == 'p' and not parent.text.strip() and pp is not None:
                parent.addprevious(include)
                pp.remove(parent)

            print "--> including file %s at line %s" % (incfn, linenum)
            if incxml.tag == 'html' and len(incxml) > 0:  # strip out outer <html> container
                for k in incxml:
                    include.addprevious(k)
            else:
                include.addprevious(incxml)
            p = include.getparent()
            if p is not None:
                p.remove(include)

    def process_includepy(self, tree):
        '''
        Handle \edXincludepy{script_file.py} inclusion of python scripts.
        '''
        self.process_include(tree, do_python=True)

    @staticmethod
    def get_filename_and_linenum(elem):
        linenum = elem.get('linenum', '<unavailable>')
        texfn = elem.get('tex_filename', elem.get('filename', '<unavailable>'))
        return "file %s line %s" % (texfn, linenum)

    def standard_error_msg(self, elem):
        msg = "Error processing element %s in %s" % (elem.tag, self.get_filename_and_linenum(elem))
        return msg

    def process_dndtex(self, tree):
        '''
        Handle \edXdndtex{dnd_file.tex} inclusion of latex2dnd tex inputs.
        '''
        tag = './/edxdndtex'
        for dndxml in tree.findall(tag):
            dndfn = dndxml.text
            linenum = dndxml.get('linenum', '<unavailable>')
            texfn = dndxml.get('filename', '<unavailable>')
            if dndfn is None:
                print "Error: %s must specify dnd tex filename!" % tag  # EVH changed 'cmd' to 'tag'
                print "See tex file %s line %s" % (texfn, linenum)
                raise
            dndfn = dndfn.strip()
            if not dndfn.endswith('.tex'):
                print "Error: dnd file %s should be a .tex file!" % dndfn
                print "See tex file %s line %s" % (texfn, linenum)
                raise
            if not os.path.exists(dndfn):
                print "Error: dnd tex file %s does not exist!" % dndfn
                print "See tex file %s line %s" % (texfn, linenum)
                raise
            try:
                dndsrc = open(dndfn).read()
            except Exception, err:
                print "Error %s: cannot open dnd tex file %s to read" % (err, dndfn)
                print "See tex file %s line %s" % (texfn, linenum)
                raise

            # Use latex2dnd to compile dnd tex into edX XML.
            #
            # For dndfile.tex, at least two files must be produced: dndfile_dnd.xml and
            # dndfile_dnd.png
            #
            # we copy all the *.png files to static/images/<dndfile>/
            #
            # run latex2dnd only when the dndfile_dnd.xml file is older than dndfile.tex

            fnb = os.path.basename(dndfn)
            fnpre = fnb[:-4]
            fndir = path(os.path.dirname(dndfn))
            xmlfn = fndir / (fnpre + '_dnd.xml')

            run_latex2dnd = False
            if not os.path.exists(xmlfn):
                run_latex2dnd = True
            if not run_latex2dnd:
                dndmt = os.path.getmtime(dndfn)
                xmlmt = os.path.getmtime(xmlfn)
                if dndmt > xmlmt:
                    run_latex2dnd = True
            if run_latex2dnd:
                options = ''
                if dndxml.get('can_reuse', 'False').lower().strip() != 'false':
                    options += '-C'
                cmd = 'cd "%s"; latex2dnd -r %s -v %s %s.tex' % (fndir, dndxml.get('resolution', 210), options, fnpre)
                print "--> Running %s" % cmd
                sys.stdout.flush()
                status = os.system(cmd)
                if status:
                    print "Oops - latex2dnd apparently failed - aborting!"
                    raise
                imdir = self.output_dir / ('static/images/%s' % fnpre)
                os.system('mkdir -p %s' % imdir)
                cmd = "cp %s/%s*.png %s/" % (fndir, fnpre, imdir)
                print "----> Copying dnd images: %s" % cmd
                sys.stdout.flush()
                status = os.system(cmd)
                if status:
                    print "Oops - copying images from latex2dnd apparently failed - aborting!"
                    raise
            else:
                print "--> latex2dnd XML file %s is up to date: %s" % (xmlfn, fnpre)

            # change dndtex tag to become include
            # change filename to become dndfile_dnd.xml
            # this will trigger an include of that XML in process_include, which happens after this filter

            dndxml.tag = 'edxinclude'
            dndxml.text = xmlfn

    def process_general_hint_system(self, tree):
        '''
        Include general_hint_system.py script for problems which have hints specified.
        '''
        mydir = os.path.dirname(__file__)
        libpath = path(os.path.abspath(mydir + '/python_lib'))
        ghsfn = libpath / 'general_hint_system.py'

        # find all instances of <edx_general_hint_system />,
        # but at most one per problem

        for problem in tree.findall('.//problem'):
            isdone = False
            for eghs in problem.findall('.//edx_general_hint_system'):
                incxml = etree.fromstring('<script><![CDATA[\n%s\n]]></script>' % open(ghsfn).read())
                if not isdone:
                    eghs.addprevious(incxml)
                    # print "  added eghs to problem %s" % problem.get('url_name')
                    isdone = True
                p = eghs.getparent()
                p.remove(eghs)

    def check_all_python_scripts(self, tree):
        '''
        Run syntax check on all python scripts
        '''
        for script in tree.findall('.//script[@type="text/python"]'):
            pyfile = tempfile.NamedTemporaryFile(mode='w', delete=False)
            if script.text is None:
                print "Warning: empty script!"
                print "Script location: %s" % etree.tostring(script)
                continue
            try:
                pyfile.write(script.text)
            except Exception as err:
                print "Error checking python script %s" % script.text
                print str(err)
                print "Script location: %s" % etree.tostring(script)
                continue
            pyfile.close()
            try:
                py_compile.compile(pyfile.name, doraise=True)
            except Exception as err:
                print "Error in python script %s! Err=%s" % (pyfile.name, err)
                print "Script location: %s" % etree.tostring(script)
                print "Aborting!"
                raise Exception(self.standard_error_msg(script))
            os.unlink(pyfile.name)

    def generate_course_unit_tests(self, xml):
        '''
        Generate course unit tests for all (suitable) answer boxes.
        The unit tests are stored as a YAML file, and may be used
        to test an edX course (on an openedx site) using the edxcut package.

        We construct the unit test set by going through all responses elements
        in the XML, and retrieving their corresponding AnswerBox objects, which
        have unit test response & expectation specifications.

        The actual unit test YAML file is generated using CourseUnitTestSet.
        '''
        responsetags = ['customresponse', 'optionresponse', 'multiplechoiceresponse', 
                        'choiceresponse', 'numericalresponse', 'formularesponse',
                        'stringresponse', 'symbolicresponse']
        cutset = CourseUnitTestSet()
        for problem in xml.findall('.//problem'):
            dn = problem.get('display_name')
            un = problem.get('url_name')
            # need to know the order of the responses, if there are multiple aboxes in a problem.
            # this is because they are submitted all together, and the edx platform uses a 
            # sequential index to number the input string IDs.
            #
            # Thus, we walk the tree, and keep orer intact

            response_elements = []
            def walk(xml):
                if xml.tag in responsetags:
                    response_elements.append(xml)
                else:
                    for elem in xml:
                        walk(elem)
            walk(problem)

            response_tests = []

            for response in response_elements:
                xmlstr = etree.tostring(response)
                abox = self.p2x.renderer.answer_box_objects.get(xmlstr.strip(), None)
                if not abox:
                    if self.verbose:
                        print "[latex2edx] generate_course_unit_tests %s: failed to find abox for response '%s'" % (un, xmlstr)
                    continue
                if self.verbose:
                    print "[latex2edx] generate_course_unit_tests %s: found abox %s" % (un, abox.aboxstr)
                
                # turn abox test list (of dicts) into list of AnswerBoxUnitTest objects
                abox_test_set = []
                count = 0
                for test in abox.tests:
                    count += 1
                    test['url_name'] = un
                    # print "test_spec=%s" % test
                    abut = AnswerBoxUnitTest(test_spec=test, test_name="%s/test_%d" % (dn, count))
                    abox_test_set.append(abut)

                response_tests.append(abox_test_set)

            # now construct the actual test cases, combining all tests for all aboxes in this problem
            all_combined_tests = []
            def make_test(all_tests, the_test=None):
                if len(all_tests)>0:
                    this_test_set = all_tests[0]
                    for a_test in this_test_set:
                        if not the_test:
                            make_test(all_tests[1:], a_test)
                        else:
                            make_test(all_tests[1:], the_test + a_test)
                else:
                    if the_test:
                        all_combined_tests.append(the_test)

            make_test(response_tests)
            if self.verbose:
                print "[latex2edx] generate_course_unit_tests adding %d tests for problem %s" % (len(all_combined_tests), un)

            count = 0
            for test in all_combined_tests:	# rename tests, since counts may have changed
                count += 1
                test.name = "(%s) %s/test_%d" % (dn, un, count)
                
            cutset.add_tests(all_combined_tests)
        cutset.output_to_file(self.output_cutset)
        print "[latex2edx] %s course unit tests output to %s" % (len(cutset.tests), self.output_cutset)

    def add_url_names(self, xml):
        '''
        Generate unique url_name database keys for all XML descriptor tags, for
        which the user did not provide one.  Do this by recursively walking the
        xml tree.
        '''
        # print "add_url_names: %s" % xml.tag
        if xml.tag in self.DescriptorTags:
            if not xml.tag == 'course':
                dn = xml.get('display_name', '')
                if not dn:
                    dn = xml.getparent().get('display_name', '') + '_' + xml.tag
                new_un = self.make_url_name(xml.get('url_name', dn), xml.tag)
                if 'url_name' in xml.keys() and not new_un == xml.get('url_name'):
                    print "Warning: url_name %s changed to %s" % (xml.get('url_name'), new_un)
                xml.set('url_name', new_un)
        if xml.tag not in ['problem', 'html']:
            for child in xml:
                self.add_url_names(child)

    def make_url_name(self, s, tag=''):
        '''
        Turn string s into a valid url_name.
        Use tag if provided.
        '''
        map = {'"\':<>': '',
               ',().;=+ ': '_',
               '&': 'and',
               '[': 'LB_',
               ']': '_RB',
               '?#* ': '_',
               u'\u2013': '-',
               u'\u2014': '-',
               }
        if not self.allow_dirs:
            map['/'] = '_'
        if not s:
            s = tag
        for m, v in map.items():
            for ch in m:
                s = s.replace(ch, v)
        if self.allow_dirs:
            # Have to do this after the rest of the mapping, as we don't want
            # ': to turn into nothing (ordering in dictionary is not guaranteed)
            s = s.replace('/', ':')
        if s in self.URLNAMES and not s.endswith(tag):
            s = '%s_%s' % (tag, s)
        while s in self.URLNAMES:
            s += 'x'
        self.URLNAMES.append(s)
        return s

    @staticmethod
    def do_attrib_string(elem):
        '''
        parse attribute strings, and add to xml elements.
        attribute strings are space delimited, and optional for elements
        like chapter, sequential, vertical, text
        '''
        attrib_string = elem.get('attrib_string', '')
        if attrib_string:
            attrib_list = split_args_with_quoted_strings(attrib_string)
            if len(attrib_list) == 1 & len(attrib_list[0].split('=')) == 1:  # a single number n is interpreted as weight="n"
                elem.set('weight', attrib_list[0])
            else:  # the normal case, can remove backwards compatibility later if desired
                for s in attrib_list:
                    attrib_and_val = s.split('=')
                    if len(attrib_and_val) != 2:
                        print "ERROR! the attribute list '%s' for element %s is not properly formatted" % (attrib_string, elem.tag)
                        # print "attrib_and_val=%s" % attrib_and_val
                        print etree.tostring(elem)
                        sys.exit(-1)
                    elem.set(attrib_and_val[0], attrib_and_val[1].strip("\""))  # remove extra quotes
        if 'attrib_string' in elem.keys():
            elem.attrib.pop('attrib_string')  # remove attrib_string

    def fix_attrib_string(self, xml):
        '''
        Convert attrib_string in <problem>, <chapter>, etc. to attributes, intelligently.
        '''
        TAGS = ['problem', 'chapter', 'sequential', 'vertical', 'course', 'html', 'video', 'discussion', 'edxdndtex',
                'conditional', 'lti']
        for tag in TAGS:
            for elem in xml.findall('.//%s' % tag):
                self.do_attrib_string(elem)

    def fix_xhtml_descriptor_in_p(self, xml):
        '''
        Sometimes have <sequential><p><problem>...</problem></p></sequential>
        Have to remove contaiing <p>
        This happens for problem, chapter, sequential, html, any DescriptorTag
        '''
        for tag in self.DescriptorTags:
            for elem in xml.findall('.//%s' % tag):
                parent = elem.getparent()
                if parent.tag == 'p':
                    for pcont in parent:
                        parent.addprevious(pcont)  # move each element in <p> up before <p>
                    parent.getparent().remove(parent)  # remove the <p>


def CommandLine():
    import pkg_resources  # part of setuptools
    version = pkg_resources.require("latex2edx")[0].version
    parser = optparse.OptionParser(usage="usage: %prog [options] filename.tex",
                                   version="%prog version " + version)
    parser.add_option('-v', '--verbose',
                      dest='verbose',
                      default=False, action='store_true',
                      help='verbose error messages')
    parser.add_option("-o", "--output-xbundle",
                      action="store",
                      dest="output_fn",
                      default="",
                      help="Filename for output xbundle file",)
    parser.add_option("-d", "--output-directory",
                      action="store",
                      dest="output_dir",
                      default="course",
                      help="Directory name for output course XML files",)
    parser.add_option("-c", "--config-file",
                      action="store",
                      dest="config_file",
                      default="latex2edx_config",
                      help="configuration file to load",)
    parser.add_option("-m", "--merge-chapters",
                      action="store_true",
                      dest="merge",
                      default=False,
                      help="merge chapters into existing course directory",)
    parser.add_option("-P", "--update-policy-file",
                      action="store_true",
                      dest="update_policy",
                      default=False,
                      help="update policy.json from settings in latex file",)
    parser.add_option("--suppress-policy-settings",
                      action="store_true",
                      dest="suppress_policy",
                      default=False,
                      help="suppress policy settings from XML files",)
    parser.add_option("--suppress-verticals",
                      action="store_true",
                      dest="suppress_verticals",
                      default=False,
                      help="do not automatically add extra verticals needed for Studio compatibility",)
    parser.add_option("-S", "--section-only",
                      action="store_true",
                      dest="section_only",
                      default=False,
                      help="export only edXsections (sequentials) -- no course or chapters",)
    parser.add_option("-x", "--xml-only",
                      action="store_true",
                      dest="xml_only",
                      default=False,
                      help="export only xbundle xml file -- no separate course content",)
    parser.add_option("--units-only",
                      action="store_true",
                      dest="units_only",
                      default=False,
                      help="export only units, including problem, html -- no course, chapter, section",)
    parser.add_option("--popups",
                      action="store_true",
                      dest="popups",
                      default=False,
                      help="enable equation and figure popup windows on clicking their references",)
    parser.add_option("--allow-directories",
                      action="store_true",
                      dest="allow_dirs",
                      default=False,
                      help="allow subdirectory structure in the xml output",)
    parser.add_option("--output-course-unit-tests",
                      action="store",
                      dest="output_cutset",
                      default="",
                      help="filename in which to output answer box unit test set (YAML format) for the course, made for testing with edxcut",)
    (opts, args) = parser.parse_args()

    if len(args) < 1:
        print 'latex2edx: wrong number of arguments'
        parser.print_help()
        sys.exit(-2)
    fn = args[0]

    config = DEFAULT_CONFIG
    extra_xml_filters = []
    # load local configuration file if available
    if os.path.exists(opts.config_file):
        import imp
        # prepend the config file's directory to the path to allow local imports inside it
        sys.path.insert(0, os.path.dirname(opts.config_file))
        cf = imp.load_source('config_file', opts.config_file)
        config.update(getattr(cf, 'local_config', {}))
        extra_xml_filters.extend(getattr(cf, 'extra_xml_filters', []))

    c = latex2edx(fn, verbose=opts.verbose, output_fn=opts.output_fn,
                  output_dir=opts.output_dir,
                  do_merge=opts.merge,
                  update_policy=opts.update_policy,
                  suppress_policy=opts.suppress_policy,
                  suppress_verticals=opts.suppress_verticals,
                  section_only=opts.section_only,
                  xml_only=opts.xml_only,
                  units_only=opts.units_only,
                  popup_flag=opts.popups,
                  allow_dirs=opts.allow_dirs,
                  output_cutset=opts.output_cutset,
                  extra_xml_filters=extra_xml_filters,
                  )
    c.convert()





import sys
import os
import re
import codecs
from logging import CRITICAL, DEBUG, INFO 
try:
    from collections import OrderedDict
except:
    from ordereddict import OrderedDict
    
from plasTeX.Renderers import XHTML
from plasTeX.TeX import TeX
from plasTeX.Renderers.PageTemplate import Renderer as _Renderer
from plasTeX.Config import config as plasTeXconfig
from xml.sax.saxutils import escape, unescape
from abox import AnswerBox, split_args_with_quoted_strings
from StringIO import StringIO

class MyRenderer(XHTML.Renderer):
    """
    PlasTeX class for rendering the latex document into XHTML + edX tags
    """
    def __init__(self, imdir='', imurl='', extra_filters=None):
        '''
        imdir = directory where images should be stored
        imurl = url base for web base location of images
        '''
        XHTML.Renderer.__init__(self)
        self.imdir = imdir
        self.imurl = imurl
        self.imfnset = []
        self.answer_box_objects = {}	# tracks AnswerBox objects, using their xmlstr repr as keys
        self.abox_config = {}	# used by AnswerBox to store state, like default config parameters

        # setup filters
        self.filters = OrderedDict()
        for ffm in self.filter_fix_math_match:
            self.filters[ffm] = self.filter_fix_math
        for ffm in self.filter_fix_displaymath_match:
            self.filters[ffm] = self.filter_fix_displaymath
        for ffm in self.filter_fix_displaymathverbatim_match:
            self.filters[ffm] = self.filter_fix_displaymathverbatim
        self.filters[self.filter_fix_abox_match] = self.filter_fix_abox
        self.filters[self.filter_fix_abox_match_with_linenum] = self.filter_fix_abox_with_linenum
        self.filters[self.filter_fix_image_match] = self.filter_fix_image
        self.filters[self.filter_fix_edxxml_match] = self.filter_fix_edxxml

        if extra_filters is not None:
            self.filters.update(extra_filters)

    filter_fix_edxxml_match = '(?s)<edxxml>\\\\edXxml{(.*?)}</edxxml>'

    @staticmethod
    def filter_fix_edxxml(m):
        xmlstr = m.group(1)
        xmlstr = xmlstr.replace('\\$ ','$')	# dollar sign must be escaped in plasTeX, but shouldn't be in XML
        xmlstr = xmlstr.replace('& ', '&')  # remove ampersand space from plasTeX
        xmlstr = xmlstr.replace('\\% ', '%')  # unescape percentage sign
        # return xmlstr
        return "<edxxml>%s</edxxml>" % xmlstr

    @staticmethod
    def fix_math_common(m, removenl=True):
        x = m.group(1).strip()
        x = x.replace('\\$ ','$')	# dollar sign must be escaped in plasTeX, but shouldn't be in XML
        x = x.replace(u'\u2019',"'")
        x = x.decode('ascii','ignore')
        x = x.replace('\\ensuremath','')
        x = x.replace('{^\\circ','{}^\\circ')	# workaround plasTeX bug
        if removenl:
            x = x.replace('\n','')
        x = escape(x)
        return x

    filter_fix_math_match  = ['(?s)<math>\$(.*?)\$</math>',
                              '(?s)<math>\\\\ensuremath{(.*?)}</math>']

    @classmethod
    def filter_fix_math(cls, m):
        x = cls.fix_math_common(m)
        if len(x)==0 or x=="\displaystyle":
            return "&nbsp;"
        return '[mathjaxinline]%s[/mathjaxinline]' % x
        
    filter_fix_displaymath_match = [r'(?s)<math>\\begin{equation}(.*?)\\end{equation}</math>',
                                    r'(?s)<math>\\begin{equation\*}(.*?)\\end{equation\*}</math>',
                                    r'(?s)<displaymath>\\begin{edXmath}(.*?)\\end{edXmath}</displaymath>',
                                    r'(?s)<math>\\\[(.*?)\\\]</math>',
                                    ]

    @classmethod
    def filter_fix_displaymath(cls, m):
        x = cls.fix_math_common(m)
        if len(x)==0 or x=="\displaystyle":
            return "&nbsp;"
        return '[mathjax]%s[/mathjax]' % x
        
    filter_fix_displaymathverbatim_match = [r'(?s)<displaymathverbatim>\\begin{edXmath}(.*?)\\end{edXmath}</displaymathverbatim>',
                                            r'(?s)<displaymathverbatim>\\edXmath(.*?)</displaymathverbatim>',
                                            ]

    @classmethod
    def filter_fix_displaymathverbatim(cls, m):
        x = escape(m.group(1).strip())
        return '[mathjax]%s[/mathjax]' % x.replace('\\end{edXmath}', '')

    filter_fix_image_match = '<includegraphics style="(.*?)">(.*?)</includegraphics>'

    def filter_fix_image(self, m):
        width = 400
        attribs = []
        print "[do_image] m=%s" % repr(m.groups())
        style = m.group(1)
        sms = style.split(',')
        for sm in sms:
            w = re.search('width=([0-9\.]+)(.*)',sm)
            if w:
                widtype = w.group(2)
                width = float(w.group(1))
                if 'in' in widtype:
                    width = width * 110
                elif 'cm' in widtype:
                    width = width * 110 / 2.54
                if '\\textwidth' in widtype:
                    width = width * 770
                width = int(width)
                if width==0:
                    width = 400
            else:
                sm = sm.strip().replace('=', ':')
                attribs.append(sm)
        attribs = ';'.join(attribs)
        if attribs:
            attribs = 'style="' + attribs + '"'

        def make_image_html(fn,k,attribs):
            self.imfnset.append(fn+k)
            # if file doesn't exist in edX web directory, copy it there
            fnbase = os.path.basename(fn)+k
            wwwfn = '%s/%s' % (self.imdir,fnbase)
            #if not os.path.exists('/home/WWW' + wwwfn):
            if 1:
                cmd = 'cp %s %s' % (fn+k,wwwfn)
                os.system(cmd)
                print cmd
                os.system('chmod og+r %s' % wwwfn)
            return '<img src="/static/%s/%s" width="%d" %s/>' % (self.imurl,
                    fnbase, width, attribs)

        fnset = [m.group(2)]
        fnsuftab = ['','.png','.pdf','.png','.jpg']
        for k in fnsuftab:
            for fn in fnset:
                if os.path.exists(fn+k):
                    if k=='.pdf':		# convert pdf to png
                        dim = width if width>400 else 400
                        # see how many pages it is
                        try:
                            npages = int(os.popen('pdfinfo %s.pdf | grep Pages:' % fn).read()[6:].strip())
                        except Exception, err:
                            # print "npages error %s" % err
                            npages = 1
                        nfound = 0
                        if npages>1:	# handle multi-page PDFs
                            fnset = ['%s-%d' % (fn,x) for x in range(npages)]
                            nfound = sum([ 1 if os.path.exists(x+'.png') else 0 for x in fnset])
                            print "--> %d page PDF, fnset=%s (nfound=%d)" % (npages, fnset, nfound)
                        if not nfound==npages:
                            os.system('convert -density 800 {fn}.pdf -scale {dim}x{dim} {fn}.png'.format(fn=fn,dim=dim))
                        if npages>1:	# handle multi-page PDFs
                            fnset = ['%s-%d' % (fn,x) for x in range(npages)]
                            print "--> %d page PDF, fnset=%s" % (npages, fnset)
                        else:
                            fnset = [fn]
                        imghtml = ''
                        for fn2 in fnset:
                            imghtml += make_image_html(fn2, '.png', attribs)
                        return imghtml
                    else:
                        return make_image_html(fn, k, attribs)
                
        fn = fnset[0]
        print 'Cannot find image file %s' % fn
        return '<img src="NOTFOUND-%s" />' % fn

    filter_fix_abox_match = r'(?s)<abox(|linenum="\d+" filename="[^>]+")>(.*?)</abox>'

    def filter_fix_abox(self, m):
        abox = AnswerBox(m.group(1), config=self.abox_config)
        self.answer_box_objects[abox.xmlstr_no_hints] = abox
        return abox.xmlstr

    filter_fix_abox_match_with_linenum = r'(?s)<abox (linenum="\d+" filename="[^>]+")>(.*?)</abox>'

    def filter_fix_abox_with_linenum(self, m):
        abox = AnswerBox(m.group(2), config=self.abox_config, context=m.group(1))
        self.answer_box_objects[abox.xmlstr_no_hints] = abox
        return abox.xmlstr

    @staticmethod
    def fix_unicode(stxt):
        ucfixset = { u'\u201d': '"',
                     u'\u2014': '&#8212;',
                     u'\u2013': '&#8211;',
                     u'\u2019': "'",
                     }

        for pre, post in ucfixset.iteritems():
            try:
                stxt = stxt.replace(pre, post)
            except Exception, err:
                print "Error in rendering (fix unicode): ",err
        return stxt

    def processFileContent(self, document, stxt):
        stxt = XHTML.Renderer.processFileContent(self, document, stxt)
        stxt = self.fix_unicode(stxt)

        for fmatch, filfun in self.filters.iteritems():
            try:
                stxt = re.sub(fmatch, filfun, stxt)
            except Exception, err:
                print "Error in rendering %s: %s" % (filfun, str(err))
                raise

        stxt = stxt.replace('<p>','<p>\n')
        stxt = stxt.replace('<li>','\n<li>')
        stxt = stxt.replace('&nbsp;','&#160;')

        stxt = stxt[stxt.index('<body>')+6:stxt.index('</body>')]

        XML_HEADER = '<document>'
        XML_TRAILER = '</document>'

        self.xhtml = XML_HEADER + stxt + XML_TRAILER
        return self.xhtml

    def cleanup(self, document, files, postProcess=None):
        res = _Renderer.cleanup(self, document, files, postProcess=postProcess)
        return res


class plastex2xhtml(object):
    '''
    Use plastex to convert .tex file to .xhtml, with special edX macros.

    This procecss requires the "render" directory, with its .zpts files, as well
    as the edXpsl.py file, with its plastex python macros.

    '''

    def __init__(self,
                 fn,
                 imdir="static/images",
                 imurl="",
                 fp=None,
                 extra_filters=None,
                 latex_string=None,
                 add_wrap=False,
                 fix_plastex_optarg_bug=True,
                 verbose=False):
        '''
        fn            = tex filename (should end in .tex)
        imdir         = directory where images are to be stored
        imurl         = web root for images
        fp            = file object (optional) - used instead of open(fn), if provided
        extra_filters = dict with key=regular exp, value=function for search-replace, for
                        post-processing of XHTML output
        latex_string  = latex string (overrides fp and fn)
        add_wrap      = if True, then assume latex is partial, and add preamble and postfix
        fix_plastex_optarg_bug = if True, then filter the input latex to fix the plastex bug 
                                 triggered e.g. by \begin{edXchapter} and \begin{edXsection}
                                 being placed with no empty newline inbetween
        verbose       = if True, then do verbose logging
        '''

        if fn.endswith('.tex'):
            ofn = fn[:-4]+'.xhtml'
        else:
            ofn = fn + ".xhtml"

        self.input_fn = fn
        self.output_fn = ofn
        self.fp = fp
        self.latex_string = latex_string
        self.add_wrap = add_wrap
        self.verbose = verbose
        self.renderer = MyRenderer(imdir, imurl, extra_filters)
        self.fix_plastex_optarg_bug = fix_plastex_optarg_bug

        # Instantiate a TeX processor and parse the input text
        tex = TeX()
        tex.ownerDocument.config['files']['split-level'] = -100
        tex.ownerDocument.config['files']['filename'] = self.output_fn
        tex.ownerDocument.config['general']['theme'] = 'plain'

        plasTeXconfig.add_section('logging')
        plasTeXconfig['logging'][''] = CRITICAL

        self.tex = tex
        if not self.verbose:
            tex.disableLogging()

    def convert(self):
        self.generate_xhtml()	# do conversion

    def generate_xhtml(self):

        if self.verbose:
            print "============================================================================="
            print "Converting latex to XHTML using PlasTeX with custom edX macros"
            print "Source file: %s" % self.input_fn
            print "============================================================================="
    
        # set the zpts templates path
        mydir = os.path.dirname(__file__)
        zptspath = os.path.abspath(mydir + '/render')
        os.environ['XHTMLTEMPLATES'] = zptspath

        # print os.environ['XHTMLTEMPLATES']

        # add our python plastex package directory to python path
        plastexpydir = os.path.abspath(mydir + '/plastexpy')
        sys.path.append(plastexpydir)

        # get the input latex file
        if self.latex_string is None:
            if self.fp is None:
                self.fp = codecs.open(self.input_fn)
            self.latex_string = self.fp.read()
            self.latex_string = self.latex_string.replace('\r','\n') # convert from mac format EOL
        
        if self.fix_plastex_optarg_bug:
            self.latex_string = self.do_fix_plastex_optarg_bug(self.latex_string)

        # add preamble and postfix wrap?
        if self.add_wrap:
            PRE = """\\documentclass[12pt]{article}\n\\usepackage{edXpsl}\n\n\\begin{document}\n\n"""
            POST = "\n\n\\end{document}"
            self.latex_string = PRE + self.latex_string + POST
 
        source = StringIO(self.latex_string)
        source.name = self.input_fn
        self.tex.input(source)
        document = self.tex.parse()
        
        self.renderer.render(document)

        print "XHTML generated (%s): %d lines" % (self.output_fn, len(self.renderer.xhtml.split('\n')))
        return self.renderer.xhtml

    @property
    def xhtml(self):
        return self.renderer.xhtml
    
    @staticmethod
    def do_fix_plastex_optarg_bug(texstring):
        '''
        PlasTeX processing appears to have a bug, 
        wherein if the tex document has two consecutive lines like this:

          \begin{edXchapter}{Basic examples}
          \begin{edXsection}{Basic example problems}

        then the \begin{edXsection} gets eaten up as an optional argument
        to \begin{edXchapter}.  This happens for all the Base.Environment
        objects defined in edXpsl.py with optional arguments.

        This can be fixed by introducing an extra newline, e.g.:

          \begin{edXchapter}{Basic examples}

          \begin{edXsection}{Basic example problems}

        or by adding a {} at the end:

          \begin{edXchapter}{Basic examples}{}
          \begin{edXsection}{Basic example problems}{}

        Here, we fix the problem by adding extra newlines after every \begin{*}
        where * is one of the known new environments with optional arguments.
        '''

        edXenvironments = ['edXcourse', 
                           'edXchapter', r'edXchapter*',
                           'edXsection', r'edXsection*',
                           'edXsequential', r'edXsequential*',
                           'edXvertical', r'edXvertical*',
                           'edXtext',
                           'edXproblem',
                           'edXsolution',
                           'edXshowhide',
                           ]

        newstring = []
        insert_nl = False

        for line in texstring.split('\n'):

            if insert_nl:
                # insert empty line if current line is not already empty line
                if not line=='':
                    newstring.append('')
                insert_nl = False

            if not r'\begin' in line:
                newstring.append(line)
                continue

            newstring.append(line)
            for env in edXenvironments:
                if line.startswith('\\begin{%s}' % env):
                    # queue up newline insertion
                    insert_nl = True

        newstring = '\n'.join(newstring)
        return newstring



'''
Test XML insertion using the `edXxml` command with example13_edxxml.tex
'''
import os
import unittest
from path import path  # needs path.py

import latex2edx as l2e
from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class TestEdxxml(unittest.TestCase):
    '''
    This class inherits the `unittest.TestCase` class and contains the methods
    `test_edxxml1` that test the proper functionality of the latex2edx
    `edXxml` command, properly parsing all text, children, and tail text
    contained therein.
    '''

    def test_edxxml1(self):
        '''
        Test the output of `latex2edx example13_edxxml.tex` for proper
        rendering of the complete XML with multiple calls in `edXxml` in a
        given line.
        '''
        testdir = path(l2e.__file__).parent / 'testtex'
        tfn = testdir / 'example13_edxxml.tex'
        print "file %s" % tfn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, tfn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            l2eout = latex2edx(nfn, output_dir=tmdir)
            l2eout.convert()

            cfn = path(tmdir) / 'html/Code_Text.xml'
            data = open(cfn).read().split('\n')
            expected = ('A matrix <code>A</code> and column vector named '
                        '<code>b</code> can be multiplied in the form '
                        '<code>A b</code> only if the number of columns '
                        'of <code>b</code> match the number of rows in '
                        '<code>A</code>. </p>')
            self.assertEqual(data[2], expected)

if __name__ == '__main__':
    unittest.main()



import os
import re
import unittest
import json
from path import path  # needs path.py
from StringIO import StringIO

import latex2edx as l2emod
from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class TestPolicy(unittest.TestCase):

    def test_policy1(self):
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example3.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)

            l2e = latex2edx(nfn, output_dir=tmdir, update_policy=True, suppress_policy=True)
            l2e.convert()

            xbfn = nfn[:-4] + '.xbundle'
            self.assertTrue(os.path.exists(xbfn))

            pfn = path(tmdir) / 'policies/2013_Fall/policy.json'
            self.assertTrue(os.path.exists(pfn))
            dat = open(pfn).read()
            policy = json.loads(dat)
            self.assertTrue('course/2013_Fall' in policy)

            self.assertTrue(policy['course/2013_Fall']['start'] == "2014-05-11T12:00")
            self.assertTrue(policy['course/2013_Fall']['end'] == "2012-08-12T00:00")
            self.assertTrue(policy['course/2013_Fall']['showanswer'] == "always")

            self.assertTrue('chapter/Unit_2' in policy)

            self.assertTrue(policy['sequential/A_second_section']['graded'] == "true")
            self.assertTrue(policy['sequential/A_second_section']['due'] == "2016-11-22T00:00")

    def test_policy2(self):
        '''
        Check for good error message, with filename and lineno
        '''
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example10_badpolicy.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)

            try:
                l2e = latex2edx(nfn, output_dir=tmdir, update_policy=True, suppress_policy=True)
                l2e.convert()
            except Exception as err:
                pass

            print "Error = %s" % str(err)
            self.assertTrue(re.search('Error processing element sequential in file .*example10_badpolicy.tex line 18', str(err)))

if __name__ == '__main__':
    unittest.main()



'''
Run unittest on xbundle.py
'''
import unittest
from lxml import etree
from latex2edx.test.util import make_temp_directory
from latex2edx.xbundle import XBundle
# ----------------------------------------------------------------------------
# tests


class TestXBundle(unittest.TestCase):
    def testRoundTrip(self):

        print "Testing XBundle round trip import -> export"
        xb = XBundle()
        cxmls = '''
<course semester="2013_Spring" course="mitx.01">
  <chapter display_name="Intro">
    <sequential display_name="Overview">
      <html display_name="Overview text">
        hello world
      </html>
    </sequential>
    <!-- a comment -->
  </chapter>
</course>
'''

        pxmls = """
<policies semester='2013_Spring'>
  <policy>x:1</policy>
</policies>
"""

        xb.set_course(etree.XML(cxmls))
        xb.add_policies(etree.XML(pxmls))
        xb.add_about_file("overview.html", "hello overview")

        xbin = str(xb)

        # test round trip
        with make_temp_directory() as tdir:
            xb.export_to_directory(tdir)
            xb2 = XBundle()
            xb2.import_from_directory(tdir + '/mitx.01')
        
        xbreloaded = str(xb2)

        self.assertEqual(xbin, xbreloaded)

if __name__ == '__main__':
    unittest.main()



import os
import unittest
from path import path  # needs path.py
from StringIO import StringIO

import latex2edx as l2emod
from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class TestHints(unittest.TestCase):

    def test_hints1(self):
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example3.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            l2e = latex2edx(nfn, output_dir=tmdir)
            l2e.convert()
            xbfn = nfn[:-4] + '.xbundle'
            self.assertTrue(os.path.exists(xbfn))

            p1fn = path(tmdir) / 'problem/p1.xml'
            self.assertTrue(os.path.exists(p1fn))
            dat = open(p1fn).read()
            self.assertIn('# General hint system for edX', dat)

            p2fn = path(tmdir) / 'problem/p2.xml'
            self.assertTrue(os.path.exists(p2fn))
            dat = open(p2fn).read()
            self.assertIn('# General hint system for edX', dat)

            cfn = path(tmdir) / 'course/2013_Fall.xml'
            self.assertTrue(os.path.exists(cfn))

if __name__ == '__main__':
    unittest.main()



import os
import unittest
from StringIO import StringIO

from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class MakeTeX(object):
    def __init__(self, tex):
        buf = """\\documentclass[12pt]{article}\n\\usepackage{edXpsl}\n\n\\begin{document}"""
        buf += tex
        buf += "\\end{document}"
        self.buf = buf

    @property
    def fp(self):
        return StringIO(self.buf)


class TestMath(unittest.TestCase):

    def test_math1(self):
        # make sure latex math expressions are turned into mathjax
        with make_temp_directory() as tmdir:
            os.chdir(tmdir)
            fp = MakeTeX(r'$\frac{\alpha}{\sqrt{1+\beta}}$').fp
            l2e = latex2edx(tmdir + '/test.tex', fp=fp, do_images=False, output_dir=tmdir)
            self.assertIn(r'<div>[mathjaxinline]\frac{\alpha}{\sqrt{1+\beta}}[/mathjaxinline]</div>', l2e.xhtml.replace(' ', ''))

    def test_math2(self):
        with make_temp_directory() as tmdir:
            os.chdir(tmdir)
            tex = r'\begin{eqnarray}\dot{Q} = \frac{A}{R_{\rm thermal}} \Delta T\end{eqnarray}'
            expect = '[mathjaxinline]\\displaystyle \\dot{Q} = \\frac{A}{R_{\\rm thermal}} \\Delta T[/mathjaxinline]'
            l2e = latex2edx(tmdir + '/test.tex', latex_string=tex, add_wrap=True,
                            do_images=False, output_dir=tmdir)
            # print l2e.p2x.renderer.__dict__
            self.assertIn(expect, l2e.xhtml)

if __name__ == '__main__':
    unittest.main()



'''
Test calls to the edXshowhide environment a proper creation of the
supporting latex2edx.js and latex2edx.css in the static directory.
'''
import os
import unittest
from lxml import etree
from path import path  # needs path.py

import latex2edx as l2e
from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class TestShowHide(unittest.TestCase):
    '''
    This class inherits the `unittest.TestCase` class and contains the method
    `test_sh1` that tests the generation of proper html and supporting files
    '''

    def test_sh1(self):
        '''
        Test the output of `latex2edx example14_showhide.tex` for proper
        formation of the showhide element.
        '''
        testdir = path(l2e.__file__).parent / 'testtex'
        tfn = testdir / 'example14_showhide.tex'
        print "file %s" % tfn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, tfn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            l2eout = latex2edx(nfn, output_dir=tmdir)
            l2eout.convert()

            jsfn = '%s/static/latex2edx.js' % tmdir
            cssfn = '%s/static/latex2edx.css' % tmdir
            self.assertTrue(os.path.exists(jsfn))
            self.assertTrue(os.path.exists(cssfn))

            tfn = '%s/html/Text.xml' % tmdir
            xml = etree.parse(tfn).getroot()
            sh = xml.find('.//div[@class="hideshowbox"]')
            self.assertEqual(sh[0].tag, 'h4')
            self.assertEqual(sh[0].get('onclick'), 'hideshow(this);')
            self.assertEqual(sh[0].text, 'Secret text')
            self.assertEqual(sh[1].get('class'), 'hideshowcontent')
            self.assertEqual(sh[1].text.strip(), 'Hidden text')
            self.assertEqual(sh[2].get('class'), 'hideshowbottom')
            lk = xml.find('.//LINK')
            self.assertTrue('latex2edx.css' in lk.get('href'))
            sc = xml.find('.//SCRIPT')
            self.assertTrue('latex2edx.js' in sc.get('src'))

if __name__ == '__main__':
    unittest.main()



# 
# General hint system for edX
#
# Hints are specified by defining a list of hint dicts.
# Each hint dict should contain the following key/value pairs:
#
#    hint    - value = hint string to display if any matches found
#
#    string  - match on string present for in the student's answer
#    symbol  - match on math symbol in the answer
#    func    - match on math function in the answer
#    isnum   - match on answer being numerical (value ignored)
#    val     - match on numerical value of student's answer
#              value = number ...or
#              value = {'expect': number, 'tolerance': tolerance}
#    magdif  - match on difference in magnitude between given and expected numerical values being too large
#              value = expected number ...or
#              value = {'expect': number, 'max': maximum_magnitude_difference_in_log10}
#    range   - match on numerical answer being within a certain range
#              value = [min, max]
#    formula - match on formula equality (via numerical sampling);
#              value = as <expr>!<variables>@<lower_range>:<upper_range>#<num_samples>
#    parens  - match on un-balanced parentheses
#              value = ignored (anything ok)
#    eval    - match on evaluated expression, which may contain calls to other hint functions
#              value = expression to evaluate, e.g. "not string('*') and string('x')"
#
# Examples:
#
#         hints = [ {'parens': '', 'hint': 'Missing parenthesis?'},
#                   {'eval': 'not string("*")',
#                    'hint': 'Please indicate multiplication explicitly using *'},
#                   {'symbol': 'L', 'hint': 'Should your answer depend on L?'},
#                   {'string': 'D^2_eg', 'hint': 'Enter D_eg^2 and not D^2_eg'},
#                   {'symbol': 'D', 'hint': 'Enter D_eg and not D'},
#                   {'eval': 'not string("hbar")', 'hint': "Are your units correct?"},
#                   {'eval': 'string("-")', 'hint': "Are your sign(s) correct?"},
#                  ]
# 
#         ch0 = HintSystem(anum=0, hints=hints2).check_hint
#
#  <hintgroup hintfn="ch0"/>
#
# Usage:
#
# Simple case, for a single input field: define the global variable "hints" with list of hint dicts
# use "check_hint" as the hintfn in the edX capa problem.
#
# Advanced case: for multiple input fields
#
# Instantiate HintSystem with configuration parameters anum=answer number, hints = list of hint dicts
# 
#        ch1 = HintSystem(anum=0, hints=hints1).check_hint
#        ch2 = HintSystem(anum=1, hints=hints2).check_hint
#
# then use ch1 and ch2 as the hintfn in the edX capa problem.

import re
import numpy
import numbers
import random

from math import log10
from functools import partial

from calc import evaluator
from calc import ParseAugmenter

#-----------------------------------------------------------------------------
# provide compare_with_tolerance and formula_test (for equation property checking)

class HintFormulaCheck(object):

    default_tolerance = '0.01%'

    def __init__(self, tolerance=None, evalfun=None):
        if tolerance is not None:
            self.default_tolerance = tolerance
        self.evalfun = evalfun or evaluator
        return
    
    def compare_with_tolerance(self, complex1, complex2, tolerance=None, relative_tolerance=False):
        """
        Compare complex1 to complex2 with maximum tolerance tol.
    
        If tolerance is type string, then it is counted as relative if it ends in %; otherwise, it is absolute.
    
         - complex1    :  student result (float complex number)
         - complex2    :  instructor result (float complex number)
         - tolerance   :  string representing a number or float
         - relative_tolerance: bool, used when`tolerance` is float to explicitly use passed tolerance as relative.
    
         Default tolerance of 1e-3% is added to compare two floats for
         near-equality (to handle machine representation errors).
         Default tolerance is relative, as the acceptable difference between two
         floats depends on the magnitude of the floats.
         (http://randomascii.wordpress.com/2012/02/25/comparing-floating-point-numbers-2012-edition/)
         Examples:
            In [183]: 0.000016 - 1.6*10**-5
            Out[183]: -3.3881317890172014e-21
            In [212]: 1.9e24 - 1.9*10**24
            Out[212]: 268435456.0
        """
        if tolerance is None:
            tolerance = self.default_tolerance

        def myabs(elem):
            if isinstance(elem, numpy.matrix):
                return numpy.sum(abs(elem))
            return abs(elem)
    
        if isinstance(tolerance, numbers.Number):
            tolerance = str(tolerance)
        if relative_tolerance:
            tolerance = tolerance * max(myabs(complex1), myabs(complex2))
        elif tolerance.endswith('%'):
            tolerance = self.evalfun(dict(), dict(), tolerance[:-1]) * 0.01
            tolerance = tolerance * max(myabs(complex1), myabs(complex2))
        else:
            tolerance = self.evalfun(dict(), dict(), tolerance)
    
        try:
            if numpy.isinf(complex1).any() or numpy.isinf(complex2).any():
                # If an input is infinite, we can end up with `abs(complex1-complex2)` and
                # `tolerance` both equal to infinity. Then, below we would have
                # `inf <= inf` which is a fail. Instead, compare directly.
                cmp = (complex1 == complex2)
                if isinstance(cmp, numpy.matrix):
                    return cmp.all()
                return cmp
            else:
                # v1 and v2 are, in general, complex numbers:
                # there are some notes about backward compatibility issue: see responsetypes.get_staff_ans()).
                # return abs(complex1 - complex2) <= tolerance
                #
                # sum() used to handle matrix comparisons
                return numpy.sum(abs(complex1 - complex2)) <= tolerance
        except Exception as err:
            print "failure in comparison, complex1=%s, complex2=%s" % (complex1, complex2)
            print "err = ", err
            raise
    
    def is_formula_equal(self, expected, given, samples, cs=True, tolerance='0.01', evalfun=None,
                         cmpfun=None, debug=False):
        '''
        expected = expression expected by instructor
        given = expression entered by student
        samples = sample string for numerical checking (see below)
        cs = case_sensitive flag
        tolerance = tolerance specification string
        evalfun = function for doing evaluation (defaults to using self.evalfun from calc2)
        cmpfun = comparison function for testing equality (defaults to compare_with_tolerance)
        debug = flag for verbosity of debugging output
    
        samples examples:
    
        samples="m_I,m_J,I_z,J_z@1,1,1,1:20,20,20,20#50"
        samples="J,m,Delta,a,h,x,mu_0,g_I,B_z@0.5,1,1,1,1,1,1,1,1:0.5,20,20,20,20,20,20,20,20#50" 
    
        matrix sampling:
    
        samples="x,y@[1|2;3|4],[0|2;4|6]:[5|5;5|5],[8|8;8|8]#50"
    
        complex numbers:
    
        "x,y,i@[1|2;3|4],[0|2;4|6],0+1j:[5|5;5|5],[8|8;8|8],0+1j#50"
        '''
        
        if evalfun is None:
            evalfun = self.evalfun
        if cmpfun is None:
            def cmpfun(a, b, tol):
                return self.compare_with_tolerance(a, b, tol)
            
        variables = samples.split('@')[0].split(',')
        numsamples = int(samples.split('@')[1].split('#')[1])
    
        def to_math_atom(sstr):
            '''
            Convert sample range atom to float or to matrix
            '''
            if '[' in sstr:
                return numpy.matrix(sstr.replace('|',' '))
            elif 'j' in sstr:
                return complex(sstr)
            else:
                return float(sstr)
    
        sranges = zip(*map(lambda x: map(to_math_atom, x.split(",")),
                           samples.split('@')[1].split('#')[0].split(':')))
        ranges = dict(zip(variables, sranges))
    
        if debug:
            print "ranges = ", ranges
    
        for i in range(numsamples):
            vvariables = {}
            for var in ranges:
                value = random.uniform(*ranges[var])
                vvariables[str(var)] = value
            if debug:
                print "vvariables = ", vvariables
            try:
                instructor_result = evalfun(vvariables, dict(), expected, case_sensitive=cs)
            except Exception as err:
                #raise Exception("is_formula_eq: vvariables=%s, err=%s" % (vvariables, str(err)))
                #raise Exception("-- %s " % str(err))
                raise Exception("Error evaluating instructor result, expected=%s, vv=%s -- %s " % (expected, vvariables, str(err)))
            try:
                student_result = evalfun(vvariables, dict(), given, case_sensitive=cs)
            except Exception as err:
                #raise Exception("is_formula_eq: vvariables=%s, err=%s" % (vvariables, str(err)))
                raise Exception("-- %s " % str(err))
                # raise Exception("Error evaluating your input, given=%s, vv=%s -- %s " % (given, vvariables, str(err)))
            #print "instructor=%s, student=%s" % (instructor_result, student_result)
            cfret = cmpfun(instructor_result, student_result, tolerance)
            if debug:
                print "comparison result = %s" % cfret
            if not cfret:
                return False
        return True
    
    def check_formula(self, expect, ans, options=None):
        '''
        expect and ans are math expression strings.
        Check for equality using random sampling.
        options should be like samples="m_I,m_J,I_z,J_z@1,1,1,1:20,20,20,20#50"!tolerance=0.3
        i.e. a sampling range for the equality testing, in the same
        format as used in formularesponse.
    
        options may also include altanswer, an alternate acceptable answer.  Example:
    
        options="samples='X,Y,i@[1|2;3|4],[0|2;4|6],0+1j:[5|5;5|5],[8|8;8|8],0+1j#50'!altanswer='-Y*X'"
    
        note that the different parts of the options string are to be spearated by a bang (!).
        '''
        #'''
        samples = None
        tolerance = '0.1%'
        acceptable_answers = [expect]
        for optstr in options.split('!'):
            if 'samples=' in optstr:
                samples = eval(optstr.split('samples=')[1])
            elif 'tolerance=' in optstr:
                tolerance = eval(optstr.split('tolerance=')[1])
            elif 'altanswer=' in optstr:
                altanswer = eval(optstr.split('altanswer=')[1])
                acceptable_answers.append(altanswer)
        if samples is None:
            return {'ok': False, 'msg': 'Oops, problem authoring error, samples=None'}
    
        # for debugging
        # return {'ok': False, 'msg': 'altanswer=%s' % altanswer}
    
        # filter the answer if the global function "ans_filter" is defined
        if 'ans_filter' in globals() or 'ans_filter' in locals():
            try:
                global ans_filter
                ans = ans_filter(ans)
            except Exception as err:
                #raise
                pass
    
        # for debuging
        # return {'ok': False, 'msg': 'ans=%s' % ans}
    
        for acceptable in acceptable_answers:
            try:
                ok = self.is_formula_equal(acceptable, ans, samples, cs=True, tolerance=tolerance,
                                           evalfun=self.evalfun, debug=False)
            except Exception as err:
                return {'ok': False, 'msg': "Sorry, could not evaluate your expression.  Error %s" % str(err)}
            if ok:
                return {'ok':True, 'msg': ''}
    
        return {'ok':ok, 'msg': ''}
    
#-----------------------------------------------------------------------------

class HintSystem(object):

    def __init__(self, anum=0, hints=None, verbose_fail=False, 
                 extra_hint_functions=None, color="orange",
                 do_not_catch_exceptions=False,
                 tolerance=None,
                 evalfun=None,
                 ):
        '''
        anum = answer_id index number, to base hint off of
        verbose_fail = flag for verbose error messages (bool)
        extra_hint_functions = None, or dict of extra hint functions and hint types
        the_hints = hints to use (defaults to global variable "hints")
        color = color to use for hints
        '''
        
        self.anum = anum
        self.verbose_fail = verbose_fail
        self.extra_hint_functions = extra_hint_functions
        self.hints = hints
        self.color = color
        self.do_not_catch_exceptions = do_not_catch_exceptions
        self.hfc = HintFormulaCheck(tolerance=tolerance, evalfun=evalfun)

    @staticmethod
    def hint_check_unbalanced_parens(ans, term):
        '''
        ans = student answer
        term = ignored
    
        Returns True if parentheses are unbalanced
        '''
        stack = []
        pushChars, popChars = "({[", ")}]"
        for c in ans :
            if c in pushChars :
                stack.append(c)
            elif c in popChars :
                if not len(stack) :
                    return True
                else:
                    stackTop = stack.pop()
                    balancingBracket = pushChars[popChars.index(c)]
                    if stackTop != balancingBracket:
                        return True
        return len(stack)>0
    
    def hint_check_formula(self, ans, term):
        '''
        ans = student answer
        term = formula to check equality with, as <expr>!<variables>@<lower_range>:<upper_range>#<num_samples>
    
        Returns True if answer equals formula
        '''
        expect, samples = term.split('!')
        options = "samples='%s'" % samples
        ret = self.hfc.check_formula(expect, ans, options=options)
        return ret['ok']
    
    @staticmethod
    def hint_check_numerical(ans, term):
        '''
        return True if ans is numerical
        '''
        try:
            x = float(ans)
            return True
        except Exception as err:
            return False
        return False

    def hint_check_val(self, ans, term):
        '''
        ans = student answer
        term = search term to look for
    
        val: check for numerical value of answer matching "term"
    
        do not worry about errors: those are caught by the caller
        '''
        if isinstance(term, dict):
            expect = term['expect']
            tolerance = term['tolerance']
        else:
            expect = term
            tolerance = '5%'
        if not type(expect)==float:
            expect = float(eval(expect))
    
        nans = float(eval(ans))
        ok = self.hfc.compare_with_tolerance(expect, nans, tolerance=tolerance)
        if ok:
            return True
        return False
    
    
    @staticmethod
    def hint_check_magdif(ans, term):
        # compute difference of ans and expected, in terms of order of magnitude
        if isinstance(term, dict):
            expect = term['expect']
            max_magdif = term['max']
        else:
            expect = term
            max_magdif = 2
        try:
            magdif = abs( log10(abs(float(ans)))-log10(abs(expect)) )
        except Exception as err:
            return False
        if magdif > max_magdif:
            return True
        return False
    
    
    @staticmethod
    def hint_check_string(ans, term):
        '''
        string: search for string in ans

        ans = student answer
        term = search term to look for

        if term is a dict, it can contain these keys:
           
           - regexp: value = regular expression to match
           - nospaces: value = string to look for in answer (after all spaces removed from ans)
    
        don't worry about errors: those are caught by the caller
        '''
        if isinstance(term, dict):
            if 'regexp' in term:
                return (re.search(term['regexp'], ans) is not None)
            elif 'nospaces' in term:
                return (term['nospaces'] in ans.replace(' ',''))
        return ans.count(term)
    
    
    @staticmethod
    def hint_check_symbol(ans, term):
        '''
        ans = student answer
        term = search term to look for
    
        sym: search for math symbol in ans
    
        don't worry about errors: those are caught by the caller
        '''
        case_sensitive = True
        # parse expression
        math_interpreter = ParseAugmenter(ans, case_sensitive)
        math_interpreter.parse_algebra()
        found = term in math_interpreter.variables_used
        # for debugging
        # print 'for %s, found=%s, variables = %s' % (ans, found, math_interpreter.variables_used)
        return found
        
    
    @staticmethod
    def hint_check_function_used(ans, term):
        '''
        ans = student answer
        term = search term to look for
    
        search for function used in ans
    
        don not worry about errors: those are caught by the caller
        '''
        case_sensitive = True
        # parse expression
        math_interpreter = ParseAugmenter(ans, case_sensitive)
        math_interpreter.parse_algebra()
        found = term in math_interpreter.functions_used
        # for debugging
        # print 'for %s, found=%s, variables = %s' % (ans, found, math_interpreter.variables_used)
        return found
        
    
    @staticmethod
    def hint_check_range(ans, term):
        '''
        ans = student answer
        term = list [bot, top] giving numerical range to check to see if answer is within
        '''
        nans = float(eval(ans))
        [bot, top] = term
        if nans < bot:
            return False
        if nans > top:
            return False
        return True
    
    
    def check_hint(self, answer_ids, student_answers, new_cmap, old_cmap, anum=None, the_hints=None):
        '''
        answer_ids = list of indexes into student_answers
        student_answers = dict of student answers
        new_cmap = new correct_map object (see capa package)
        old_cmap = old correct_map object (see capa package)
        '''
    
        if the_hints is None:
            if self.hints is None:
                global hints
                the_hints = hints
            else:
                the_hints = self.hints
    
        # hints should be a list of dicts.  If it's a dict, then the keys give the answer number
        # and each of those dict elements should be called one at a time.
        if isinstance(the_hints, dict):
            for anum, hint in the_hints.items():
                self.check_hint(answer_ids, student_answers, new_cmap, old_cmap, anum=anum, the_hints=hint)
                # print "--> anum=%s, hint=%s, ncm=%s" % (anum, hint, new_cmap.hints)	# for debugging
            return

        if anum is None:
            anum = self.anum
        try:
            aid = answer_ids[anum]
        except Exception as err:
            raise Exception('cannot get answer_ids[%d], answer_ids=%s, new_cmap=%s, err=%s' % (self.anum, answer_ids, new_cmap, err))
    
        ans = student_answers[aid]
    
        # for debugging
        #new_cmap.set_hint_and_mode(aid, "hello world", 'always')
        #return
    
        htypes = {'val': partial(self.hint_check_val, ans),
                  'range': partial(self.hint_check_range, ans),
                  'magdif': partial(self.hint_check_magdif, ans),
                  'string': partial(self.hint_check_string, ans),
                  'symbol': partial(self.hint_check_symbol, ans),
                  'func': partial(self.hint_check_function_used, ans),
                  'formula': partial(self.hint_check_formula, ans),
                  'parens': partial(self.hint_check_unbalanced_parens, ans),
                  'isnum': partial(self.hint_check_numerical, ans),
                  'debug': None,
                  'eval': None,
              }
        if self.extra_hint_functions is not None:
            for key, ehf in self.extra_hint_functions.items():
                htypes[key] = partial(ehf, ans)
    
        # print "using the_hints = %s" % the_hints
        the_hint = None
        for hintinfo in the_hints:
            for htype, hfun in htypes.items():
                if htype in hintinfo:
                    try:
                        term = hintinfo[htype]
                        if htype=='eval':
                            ret = eval(term, htypes)	# evaluate the expression - can have function calls in it!
                        elif htype=='debug':
                            ret = True
                        else:
                            ret = hfun(term)
                        if htype=='debug':
                            hintinfo['hint'] = "Answer submitted=%s" % ans
                        if ret:
                            the_hint = hintinfo['hint']
                            if '<font' not in the_hint:
                                the_hint = ('<font color="%s">' % self.color) + the_hint + '</font>'
                            # return on first matching hint
                            new_cmap.set_hint_and_mode(aid, the_hint, 'always')
                            return
                    except Exception as err:
                        if self.do_not_catch_exceptions:
                            raise
                        if self.verbose_fail:
                            raise Exception("Error %s checking hint %s ans=%s, term=%s" % (err, htype, ans, term))





'''
Test latex2edx export to directory tree with optionally specified nested
file structure in the url_name field of problem elements.
'''
import os
import unittest
from lxml import etree
from path import path  # needs path.py

import latex2edx as l2e
from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class TestAllow_Dirs(unittest.TestCase):
    '''
    This class contains two test methods:
    `test_allow_dirs1` checks the behavior of the output when the
    allow-directories flag is not set
    `test_allow_dirs2` checks to make sure nested directories are created
    when desired
    '''

    def test_allow_dirs1(self):
        '''
        Test that the output problem is properly renamed when allow-directories
        is set to false
        '''
        testdir = path(l2e.__file__).parent / 'testtex'
        tfn = testdir / 'example15_directories.tex'
        print "file %s" % tfn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, tfn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            l2eout = latex2edx(nfn, output_dir=tmdir, allow_dirs=False)
            l2eout.convert()

            opfn = path(tmdir) / 'problem/test_subdirprob.xml'
            self.assertTrue(os.path.exists(opfn))

            cfn = path(tmdir) / 'chapter/One.xml'
            xml = etree.fromstring(open(cfn).read())
            prob = xml.find('.//problem')
            self.assertEqual(prob.get('url_name'), 'test_subdirprob')

    def test_allow_dirs2(self):
        '''
        Test that the output problem is properly nested when allow-directories
        is specified
        '''
        testdir = path(l2e.__file__).parent / 'testtex'
        tfn = testdir / 'example15_directories.tex'
        print "file %s" % tfn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, tfn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            l2eout = latex2edx(nfn, output_dir=tmdir, allow_dirs=True)
            l2eout.convert()

            opfn = path(tmdir) / 'problem/test/subdirprob.xml'
            self.assertTrue(os.path.exists(opfn))

            cfn = path(tmdir) / 'chapter/One.xml'
            xml = etree.fromstring(open(cfn).read())
            prob = xml.find('.//problem')
            self.assertEqual(prob.get('url_name'), 'test:subdirprob')

if __name__ == '__main__':
    unittest.main()



from general_hint_system import *

#-----------------------------------------------------------------------------
# py.test unit tests
    
class TestClass(object):

    class correct_map(object):
        def __init__(self):
            self.hints = {}
            return
        def set_hint_and_mode(self, aid, the_hint, mode):
            self.hints[aid] = the_hint
            self.mode = mode

    hfc = HintFormulaCheck()

    def test_hint_string_and_eval(self):
        hints = [  {'string': 'H', 
                    'hint': 'Your answer should not have H'
                },
                   {'eval': "not string('*')", 
                    'hint': 'Remember to explicitly indicate multiplication with *'
                },
               ]
        HS = HintSystem(hints=hints)
        check_hint = HS.check_hint
        aids = [0]
        answers = ['Hello']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        check_hint(aids, answers, ncmap, ocmap)
        assert('should not have H' in ncmap.hints[0])
        assert('explicitly indicate' not in ncmap.hints[0])

        answers = ['ab+cd']
        check_hint(aids, answers, ncmap, ocmap)
        assert('explicitly indicate' in ncmap.hints[0])

    def test_formula_test(self):
        expect, samples = 'x+2!x@1:10#20'.split('!')
        options = "samples='%s'" % samples
        ans = 'x+2'
        ret = self.hfc.check_formula(expect, ans, options=options)
        if not ret['ok']:
            print ret
        assert(ret['ok'])

    def test_check_formula(self):
        HS = HintSystem()
        term = 'x+2!x@1:10#20'
        ans = 'x+2'
        ret = HS.hint_check_formula(ans, term)
        assert(ret)

    def test_hint_formula(self):
        hints = [  {'formula': 'x+2!x@1:10#20', 
                    'hint': 'missing two?'
                },
               ]
        HS = HintSystem(hints=hints)
        check_hint = HS.check_hint
        HS.verbose_fail = False
        aids = [0]
        answers = ['Hello']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        check_hint(aids, answers, ncmap, ocmap)
        assert(len(ncmap.hints)==0)

        answers = ['x+2']
        HS.verbose_fail = True
        check_hint(aids, answers, ncmap, ocmap)
        assert('missing two' in ncmap.hints[0])

        answers = ['-(x+2-2*x)+4']	# still x+2
        HS.verbose_fail = True
        check_hint(aids, answers, ncmap, ocmap)
        assert('missing two' in ncmap.hints[0])

    def test_hint_val(self):
        hints = [  {'val': {'expect': '12', 'tolerance': '5%'}, 
                    'hint': 'a hint'
                },
               ]
        HS = HintSystem(hints=hints)
        check_hint = HS.check_hint
        HS.verbose_fail = False
        aids = [0]
        answers = ['Hello']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        check_hint(aids, answers, ncmap, ocmap)
        assert(len(ncmap.hints)==0)

        answers = ['12*1.04']
        check_hint(aids, answers, ncmap, ocmap)
        print ncmap.hints
        assert('a hint' in ncmap.hints[0])

    def test_hint_string2(self):
        hints = [  {'string': {'regexp': 'ab.*cd'}, 'hint': 'a hint' },
                   {'string': {'nospaces': 'xyz'}, 'hint': 'another hint' },
               ]
        HS = HintSystem(hints=hints)
        check_hint = HS.check_hint
        HS.verbose_fail = False
        aids = [0]
        answers = ['Hello']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        check_hint(aids, answers, ncmap, ocmap)
        assert(len(ncmap.hints)==0)

        answers = ['cxab syy cd13']
        check_hint(aids, answers, ncmap, ocmap)
        print ncmap.hints
        assert('a hint' in ncmap.hints[0])

        answers = ['asdx y  zbb']
        check_hint(aids, answers, ncmap, ocmap)
        print ncmap.hints
        assert('another hint' in ncmap.hints[0])

    def test_hint_val2(self):
        anum0 = 1/((2*16.3)**2)

        hints = [
            {'parens': '', 'hint': 'Missing parenthesis?'},
            {'val': {'expect': anum0*4, 'tolerance': '10%'}, 'hint': 'Off by factor of two somewhere?'},
            {'magdif': anum0, 'hint': 'You are off by more than two orders of magnitude'},
            ]
        HS = HintSystem(hints=hints)
        check_hint = HS.check_hint
        HS.verbose_fail = False
        aids = [0]
        answers = ['Hello']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        check_hint(aids, answers, ncmap, ocmap)
        assert(len(ncmap.hints)==0)

        answers = ['1/(16.3*16.3)']
        check_hint(aids, answers, ncmap, ocmap)
        print ncmap.hints
        assert('factor of two' in ncmap.hints[0])

    def test_hint_symbol(self):
        hints = [  {'symbol': 'hbar', 
                    'hint': 'missing hbar'
                },
               ]
        HS = HintSystem(hints=hints)
        check_hint = HS.check_hint
        aids = [0]
        answers = ['Hello']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        check_hint(aids, answers, ncmap, ocmap)
        assert(len(ncmap.hints)==0)

        answers = ['x+2']
        HS.verbose_fail = True
        check_hint(aids, answers, ncmap, ocmap)
        assert(len(ncmap.hints)==0)

        answers = ['2+hbar/3']
        HS.verbose_fail = True
        check_hint(aids, answers, ncmap, ocmap)
        assert('missing hbar' in ncmap.hints[0])

        answers = ['2+hbar_3/3']
        ncmap = self.correct_map()
        HS.verbose_fail = True
        check_hint(aids, answers, ncmap, ocmap)
        assert(len(ncmap.hints)==0)

    def test_hint_function(self):
        hints = [  {'symbol': 'hbar', 
                    'hint': 'missing hbar'
                },
                {'func': 'cos', 'hint': 'oscillates'},
               ]
        HS = HintSystem(hints=hints)
        check_hint = HS.check_hint
        aids = [0]
        answers = ['Hello']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        check_hint(aids, answers, ncmap, ocmap)
        assert(len(ncmap.hints)==0)

        answers = ['y + sqrt(cos(x))']
        HS.verbose_fail = True
        check_hint(aids, answers, ncmap, ocmap)
        assert('oscillates' in ncmap.hints[0])

    def test_hint_magdif(self):
        hints = [  {'magdif': {'expect': 3.141, 'max': 2}, 
                    'hint': 'You are more than two orders of magnitude off'
                },
               ]
        HS = HintSystem(hints=hints)
        check_hint = HS.check_hint
        aids = [0]
        answers = ['Hello']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        check_hint(aids, answers, ncmap, ocmap)
        assert(len(ncmap.hints)==0)

        answers = ['3e8']
        HS.verbose_fail = True
        check_hint(aids, answers, ncmap, ocmap)
        assert('two orders' in ncmap.hints[0])

    def test_hint_range(self):
        hints = [  {'range': [4, 9],
                    'hint': 'You are out of range'
                },
               ]
        HS = HintSystem(hints=hints)
        check_hint = HS.check_hint
        HS.verbose_fail = False
        aids = [0]
        answers = ['Hello']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        check_hint(aids, answers, ncmap, ocmap)
        assert(len(ncmap.hints)==0)

        answers = ['3e8']
        HS.verbose_fail = True
        check_hint(aids, answers, ncmap, ocmap)
        assert(len(ncmap.hints)==0)

        answers = ['5']
        HS.verbose_fail = True
        check_hint(aids, answers, ncmap, ocmap)
        assert('out of range' in ncmap.hints[0])

    def test_hint_parens(self):
        hints = [  {'parens': '',
                    'hint': 'Missing parenthesis?'
                },
               ]
        HS = HintSystem(hints=hints)
        check_hint = HS.check_hint
        HS.verbose_fail = False
        aids = [0]
        answers = ['Hello']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        check_hint(aids, answers, ncmap, ocmap)
        assert(len(ncmap.hints)==0)

        answers = ['(a+23']
        HS.verbose_fail = True
        HS.do_not_catch_exceptions = True
        check_hint(aids, answers, ncmap, ocmap)
        assert('Missing' in ncmap.hints[0])

        answers = ['2+(3*2+(a+x)']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        check_hint(aids, answers, ncmap, ocmap)
        assert('Missing' in ncmap.hints[0])

    def test_hint_isnum(self):
        hints = [  {'isnum': '',
                    'hint': 'not numerical'
                },
               ]
        HS = HintSystem(hints=hints)
        check_hint = HS.check_hint
        HS.verbose_fail = False
        aids = [0]
        answers = ['Hello']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        check_hint(aids, answers, ncmap, ocmap)
        assert(len(ncmap.hints)==0)

        answers = ['23']
        HS.verbose_fail = True
        HS.do_not_catch_exceptions = True
        check_hint(aids, answers, ncmap, ocmap)
        assert('numerical' in ncmap.hints[0])

    def test_hint_symbol2(self):
        '''
        different hints for two different answer boxes
        '''
        hints1 = [  {'string': 'hbar', 'hint': 'why hbar'}, ]
        hints2 = [  {'string': 'hbar', 'hint': 'hello hbar'}, ]
        aids = [0, 1]
        answers = ['Hello hbar', 'world hbar']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        ch1 = HintSystem(anum=0, hints=hints1).check_hint
        ch2 = HintSystem(anum=1, hints=hints2).check_hint
        ch1(aids, answers, ncmap, ocmap)
        print ncmap.hints
        assert('why' in ncmap.hints[0])

        ncmap = self.correct_map()
        ocmap = self.correct_map()
        ch2(aids, answers, ncmap, ocmap)
        assert('hello' in ncmap.hints[1])

    def test_complex_hint1(self):
        '''
        sequence of hints
        '''
        hints2 = [ {'parens': '', 'hint': 'Missing parenthesis?'},
                   {'eval': 'not string("*")',
                    'hint': 'Please indicate multiplication explicitly using *'},
                    {'symbol': 'L', 'hint': 'Should your answer depend on L?'},
                    {'string': 'D^2_eg', 'hint': 'Enter D_eg^2 and not D^2_eg'},
                    {'symbol': 'D', 'hint': 'Enter D_eg and not D'},
                    {'eval': 'not string("hbar")', 'hint': "Are your units correct?"},
                    {'eval': 'string("-")', 'hint': "Are your sign(s) correct?"},
                    ]
        ch2 = HintSystem(anum=0, hints=hints2).check_hint
        aids = [0]
        answers = ['E_0^2*D_eg^2/(2*Delta)']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        ch2(aids, answers, ncmap, ocmap)
        print ncmap.hints[0]
        assert('units correct' in ncmap.hints[0])
        
    def test_multiple_hints1(self):
        '''
        different hints for two different answer boxes, with one hints dict
        '''
        my_hints = {0: [  {'string': 'hbar', 'hint': 'why hbar'}, ],
                    1: [  {'string': 'quux', 'hint': 'hello quux'}, ],
                    }
        aids = [0, 1]
        answers = ['Hello hbar', 'world quux']
        ncmap = self.correct_map()
        ocmap = self.correct_map()
        ch = HintSystem(hints=my_hints).check_hint
        ch(aids, answers, ncmap, ocmap)
        print ncmap.hints
        assert('why' in ncmap.hints[0])
        assert('hello' in ncmap.hints[1])




import os
import contextlib
import unittest
import tempfile
import shutil
from path import path  # needs path.py
from lxml import etree
from StringIO import StringIO

import latex2edx as l2emod
from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class TestVideo(unittest.TestCase):

    def test_video1(self):
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example4.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            l2e = latex2edx(nfn, output_dir=tmdir)
            l2e.convert()
            xbfn = nfn[:-4] + '.xbundle'
            self.assertTrue(os.path.exists(xbfn))
            # xb = open(xbfn).read()

            cfn = path(tmdir) / 'chapter/Unit_2.xml'
            self.assertTrue(os.path.exists(cfn))

            xml = etree.parse(cfn).getroot()
            assert(xml.tag == 'chapter')
            assert(xml.get('display_name') == "Unit 2")
            assert(xml.get('start') == "2013-11-22")
            assert(xml[0].tag == 'sequential')
            assert(xml[0][0].tag == 'vertical')
            assert(xml[0][0][0].tag == 'video')
            assert(xml[0][0][0].get('url_name') == 'A_sample_video')
            assert(not xml.findall('.//p'))
            assert(xml[0][1][0].tag == 'problem')
            assert(xml[0][2][0].tag == 'discussion')
            assert(xml[0][2][0].get('url_name') == 'Discuss_this_question')
            assert(len(xml[0]) == 3)

            cfn = path(tmdir) / 'video/A_sample_video.xml'
            self.assertTrue(os.path.exists(cfn))
            self.assertTrue('<video display_name="A sample video" youtube_id_1_0="u23ZUSu7-HY" source="test"/>' in open(cfn).read())

            cfn = path(tmdir) / 'discussion/Discuss_this_question.xml'
            self.assertTrue(os.path.exists(cfn))
            self.assertTrue('<discussion display_name="Discuss this question" forumid="discuss2"/>' in open(cfn).read())

if __name__ == '__main__':
    unittest.main()



import os
import contextlib
import unittest
import tempfile
import shutil
from path import path  # needs path.py
from StringIO import StringIO

import latex2edx as l2emod
from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class TestBad_Abox(unittest.TestCase):

    def test_bad_abox1(self):
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example8_badscript.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            try:
                l2e = latex2edx(nfn, output_dir=tmdir)
                l2e.convert()
                err = ""
            except Exception as err:
                print err
            assert('abox located: linenum="43"' in str(err))
            xbfn = nfn[:-4] + '.xbundle'
            self.assertTrue(not os.path.exists(xbfn))

if __name__ == '__main__':
    unittest.main()



import os
import unittest
from path import path  # needs path.py
from StringIO import StringIO

import latex2edx as l2emod
from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class TestOutput_Fmts(unittest.TestCase):

    def test_output_fmts1(self):
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example6.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            try:
                l2e = latex2edx(nfn, output_dir=tmdir, xml_only=True)
                l2e.convert()
                err = ""
            except Exception as err:
                print err

            xbfn = nfn[:-4] + '.xbundle'
            self.assertTrue(os.path.exists(xbfn))

            cfn = path(tmdir) / 'course/2013_Fall.xml'
            self.assertTrue(not os.path.exists(cfn))

    def test_output_fmts2(self):
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example9_section_only.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            try:
                l2e = latex2edx(nfn, output_dir=tmdir, section_only=True)
                l2e.convert()
                err = ""
            except Exception as err:
                print err

            xbfn = nfn[:-4] + '.xbundle'
            self.assertTrue(os.path.exists(xbfn))

            cfn = path(tmdir) / 'course/2013_Fall.xml'
            self.assertTrue(not os.path.exists(cfn))

            cfn = path(tmdir) / 'sequential/A_second_section.xml'
            os.system('/bin/ls -R %s' % (path(tmdir)))
            self.assertTrue(os.path.exists(cfn))

            cfn = path(tmdir) / 'problem/p1.xml'
            self.assertTrue(os.path.exists(cfn))

            cfn = path(tmdir) / 'vertical/A_second_section_vertical.xml'
            self.assertTrue(os.path.exists(cfn))

    def test_output_fmts3(self):
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example9_section_only.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            try:
                l2e = latex2edx(nfn, output_dir=tmdir, section_only=True, suppress_verticals=True)
                l2e.convert()
                err = ""
            except Exception as err:
                print err

            xbfn = nfn[:-4] + '.xbundle'
            self.assertTrue(os.path.exists(xbfn))

            cfn = path(tmdir) / 'course/2013_Fall.xml'
            self.assertTrue(not os.path.exists(cfn))

            cfn = path(tmdir) / 'sequential/A_second_section.xml'
            os.system('/bin/ls -R %s' % (path(tmdir)))
            self.assertTrue(os.path.exists(cfn))

            cfn = path(tmdir) / 'problem/p1.xml'
            self.assertTrue(os.path.exists(cfn))

            cfn = path(tmdir) / 'vertical/A_second_section_vertical.xml'
            self.assertTrue(not os.path.exists(cfn))

    def test_output_fmts4(self):
        '''
        units only (problems)
        '''
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example1.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            try:
                l2e = latex2edx(nfn, output_dir=tmdir, units_only=True)
                l2e.convert()
                err = ""
            except Exception as err:
                print err

            xbfn = nfn[:-4] + '.xbundle'
            self.assertTrue(os.path.exists(xbfn))

            cfn = path(tmdir) / 'course/2013_Fall.xml'
            self.assertTrue(not os.path.exists(cfn))

            cfn = path(tmdir) / 'sequential'
            self.assertTrue(not os.path.exists(cfn))

            cfn = path(tmdir) / 'problem/Problem_2.xml'
            self.assertTrue(os.path.exists(cfn))

            cfn = path(tmdir) / 'vertical'
            self.assertTrue(not os.path.exists(cfn))

if __name__ == '__main__':
    unittest.main()



import os
import re
import unittest
from path import path  # needs path.py
from lxml import etree
from StringIO import StringIO

import latex2edx as l2emod
from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class TestedXinclude(unittest.TestCase):

    def test_edXinclude1(self):
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example7.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            l2e = latex2edx(nfn, output_dir=tmdir)
            l2e.convert()
            xbfn = nfn[:-4] + '.xbundle'
            self.assertTrue(os.path.exists(xbfn))
            # xb = open(xbfn).read()

            cfn = path(tmdir) / 'chapter/Unit_2.xml'
            assert(os.path.exists(cfn))
            self.assertTrue(os.path.exists(cfn))
            data = open(cfn).read()
            expect = r"""<vertical url_name="A_second_section_vertical1">
      <staffgrading url_name="project_paper"/>
    </vertical>"""
            assert(expect in data)

            cfn = path(tmdir) / 'staffgrading/project_paper.xml'
            assert(os.path.exists(cfn))

    def test_edXinclude2(self):
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example7_bad.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)

            try:
                l2e = latex2edx(nfn, output_dir=tmdir)
                l2e.convert()
            except Exception as err:
                pass

            print "Error = %s" % str(err)
            self.assertTrue(re.search('Error processing element edxincludepy in file .*example.*\.tex line 25', str(err)))

if __name__ == '__main__':
    unittest.main()



'''
Test calls to \index{} and the creation of a key_map.json file for use as
a searchable index, currently only seems to work as a static page.
'''
import json
import os
import unittest
from path import path  # needs path.py

import latex2edx as l2e
from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class TestKey(unittest.TestCase):
    '''
    This class inherits the `unittest.TestCase` class and contains the method
    `test_key1` that test the proper functionality of the latex2edx `index`
    command.
    '''

    def test_key1(self):
        '''
        Test the output of `latex2edx example12_index.tex` for proper
        formation of the key_map.json file.
        '''
        testdir = path(l2e.__file__).parent / 'testtex'
        tfn = testdir / 'example12_index.tex'
        print "file %s" % tfn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, tfn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.chdir(tmdir)
            l2eout = latex2edx(nfn, output_dir=tmdir)
            l2eout.convert()

            jsfn = '%s/static/key_map.json' % tmdir
            keymap = json.load(open(jsfn))
            self.assertEqual(set(keymap.keys()), set(['topic', 'numerical']))
            self.assertEqual(keymap['topic'][0],
                             ['Module_1/A_lecture_section/1',
                              'Module_1/Another_lecture_section/1'])
            self.assertEqual(keymap['numerical'],
                             [['Module_0/A_problem_section/2'],
                              ['Example problem']])

if __name__ == '__main__':
    unittest.main()



'''
Representation of unit tests for answer boxes in edX-platform courses, together with a
representation of a full set of such tests for a course.
'''

import yaml
import os

class AnswerBoxUnitTest(object):
    '''
    Representation of a single unit test for an answer box.
    '''
    SPEC_FIELDS = ['url_name', 'responses', 'expected']
    def __init__(self, test_spec=None, test_name=None):
        '''
        test_spec = dict giving url_name, responses, and expected.
        test_name = name of this test (or, e.g. at least a sequential index number for the test)

        responses   = ordered list of strings, specifying user input for each input in the answer box element
        expected    = either list or single string instance of "correct" or "incorrect" or "error"
        box_indexes = list of (x,y) integer pairs specifying index number of the answer box
                      entry.  This is used to construct the input box ID, which is of the form
                      input_<url_name>_<x>_<y>, where <x> indexes which \abox the input is, and 
                      <y> indexes which input element it is, within a given abox (for aboxes with
                      multiple input boxes).  This list should have the same length as the responses.
                      If box_indices is not provided, then it defaults to 
                                  zip(range(len(responses)), [0]*len(responses))
                      Note that the edX platform seems to start x from 2 (and not 1, or 0), and
                      y from 1.  We stick to the convention that x and y both start at 0, and leave
                      the offsets for edxapi.
        '''
        self.url_name = None
        self.responses = []
        self.expected = []
        self.name = test_name
        self.box_indexes = []
        if test_spec:
            for field in self.SPEC_FIELDS:
                if not field in test_spec:
                    print "Missing %s from test %s" % (field, test_name)
                else:
                    setattr(self, field, test_spec[field])
            self.box_indexes = test_spec.get('box_indexes', self.box_indexes)
        if not isinstance(self.responses, list):
            raise Exception("[AnswerBoxUnitTes] illegel responses=%s -- must be list" % self.responses)
        if isinstance(self.expected, list) and not len(self.expected)==len(self.responses):
            raise Exception("[AnswerBoxUnitTes] mismatched lengths responses=%s ; expected=%s" % (self.responses, self.expected))
        if not self.box_indexes:
            self.box_indexes = zip(range(len(self.responses)), [0]*len(self.responses))
        if not len(self.box_indexes)==len(self.responses):
            raise Exception("[AnswerBoxUnitTes] mismatched lengths responses=%s ; box_indexes=%s" % (self.responses, self.box_indexes))

    def __unicode__(self):
        return "[AnswerBoxUnitTest] url_name=%s, responses=%s, expected=%s indexes=%s (%s)" % (self.url_name,
                                                                                               self.responses,
                                                                                               self.expected,
                                                                                               self.box_indexes,
                                                                                               self.name)

    __str__ = __unicode__

    @property
    def expected_as_list(self):
        if not isinstance(self.expected, list):
            return [self.expected] * len(self.responses)
        return self.expected

    def box_indexes_plus(self, offset=1):
        '''
        Return box indexes, with the <x> coordinate incremented by offset
        '''
        return [(x+offset, y) for (x,y) in self.box_indexes]

    @property
    def box_indexes_as_list(self):
        '''
        Return box indexes, as list of lists (instead of as list of tuples).
        Needed so YAML outputs correctly.
        '''
        return [list(x) for x in self.box_indexes]

    @property
    def max_box_x_index(self):
        '''
        Return largest x index of box indexes
        '''
        return max([x[0] for x in self.box_indexes])

    def __add__(self, other):
        assert self.url_name==other.url_name
        both_responses = self.responses + other.responses
        both_expected = self.expected_as_list + other.expected_as_list
        both_name = "combination of %s and %s" % (str(self.name), str(other.name))
        both_indexes = self.box_indexes + other.box_indexes_plus(offset=self.max_box_x_index+1)
        return AnswerBoxUnitTest(dict(url_name=self.url_name,
                                      responses=both_responses,
                                      expected=both_expected,
                                      box_indexes=both_indexes,
                                      name=both_name))

    def as_dict(self):
        '''
        Return dict representation of this AnswerBoxUnitTest object
        '''
        data = dict(url_name=self.url_name, responses=self.responses, expected=self.expected, 
                    box_indexes=self.box_indexes_as_list)
        if self.name:
            data['name'] = self.name
        return data
        

class CourseUnitTestSet(object):
    '''
    Set of tests (AnswerBoxUnitTest objecs), for an edX-platform course.
    '''
    def __init__(self, fn=None, verbose=True, yaml_string=None):
        '''
        fn = name of file to load course unit tests from (defaults to looking for YAML)
        '''
        self.config = {}
        self.tests = []
        self.verbose = verbose
        if fn or yaml_string:
            self.load_tests_from_file(fn=fn, yaml_string=yaml_string)

    def add_tests(self, tests):
        '''
        tests - list of AnswerBoxUnitTest objects
        '''
        for test in tests:
            self.add_test(test)

    def add_test(self, test):
        '''
        test - AnswerBoxUnitTest object
        '''
        if not isinstance(test, AnswerBoxUnitTest):
            raise Exception("[CourseUnitTestSet] add_test: test must be an instance of AnswerBoxUnitTest")
        self.tests.append(test)

    def output_to_file(self, ofn):
        '''
        Write test set to output file in YAML format.
        '''
        cut_spec = {'config': self.config, 'tests': [x.as_dict() for x in self.tests]}
        open(ofn, 'w').write(yaml.dump(cut_spec))

    def load_tests_from_file(self, fn=None, yaml_string=None):
        if fn:
            if not os.path.exists(fn):
                raise Exception("[CourseUnitTestSet] Expecting course unit test config file - but no such file %s" % fn)
            yaml_string = open(fn).read()
        if not yaml_string:
            raise Exception("[CourseUnitTestSet] empty YAML string %s" % yaml_string)
        cut_specs = yaml.load(yaml_string)
        if 'config' in cut_specs:
            self.config = cut_specs['config']
            # self.__dict__.update(cut_specs['config'])
        self.cut_specs = cut_specs
        cnt = 0
        for test in cut_specs.get('tests', []):
            cnt += 1
            abutest = AnswerBoxUnitTest(test, cnt)
            self.tests.append(abutest)
        self.ntests = len(self.tests)
        if self.verbose:
            print "[CourseUnitTestSet] Loaded %s answer box unit tests from %s" % (self.ntests, fn)
            
#-----------------------------------------------------------------------------
# unit tests

def test_abut1():
    abut1 = AnswerBoxUnitTest(dict(responses=["1"], expected="correct", url_name="x"))
    abut2 = AnswerBoxUnitTest(dict(responses=["2"], expected="incorrect", url_name="x"))
    abut3 = abut1 + abut2
    assert abut3.url_name=="x"
    assert abut3.expected==["correct", "incorrect"]
    assert abut3.responses==["1", "2"]

def test_cutset1():
    yaml = """config: {a: 2}
tests:
- expected: [correct]
  name: A problem/test_1
  responses: [red]
  url_name: a_problem
    """
    cutset = CourseUnitTestSet(yaml_string=yaml)
    assert len(cutset.tests)==1
    assert cutset.config['a']==2
    assert cutset.tests[0].expected==['correct']
    assert cutset.tests[0].responses==['red']
    assert cutset.tests[0].url_name=="a_problem"

    abut2 = AnswerBoxUnitTest(dict(responses=["2"], expected="incorrect", url_name="x"))
    cutset.add_tests([abut2])
    assert len(cutset.tests)==2
    
def test_cutset2():
    yaml = """config: {a: 2}
tests:
- expected: [correct]
  name: A problem/test_1
  responses: [red]
  url_name: a_problem
    """
    cutset = CourseUnitTestSet(yaml_string=yaml)
    
    cfn = "/tmp/tmp_cutset.yaml"
    cutset.output_to_file(cfn)

    cutset2 = CourseUnitTestSet(cfn)
    assert len(cutset2.tests)==1
    assert cutset2.config['a']==2
    assert cutset2.tests[0].expected==['correct']
    assert cutset2.tests[0].responses==['red']
    assert cutset2.tests[0].url_name=="a_problem"

    



import os
import unittest
from path import path  # needs path.py
from lxml import etree
from StringIO import StringIO

import latex2edx as l2emod
from latex2edx.main import latex2edx
from latex2edx.test.util import make_temp_directory


class TestedXmath(unittest.TestCase):

    def test_edXmath1(self):
        testdir = path(l2emod.__file__).parent / 'testtex'
        fn = testdir / 'example6.tex'
        print "file %s" % fn
        with make_temp_directory() as tmdir:
            nfn = '%s/%s' % (tmdir, fn.basename())
            os.system('cp %s/* %s' % (testdir, tmdir))
            os.mkdir('%s/dnd' % tmdir)
            os.system('cp %s/quadratic.tex %s/dnd/' % (testdir, tmdir))
            os.chdir(tmdir)
            l2e = latex2edx(nfn, output_dir=tmdir)
            l2e.convert()
            xbfn = nfn[:-4] + '.xbundle'
            self.assertTrue(os.path.exists(xbfn))
            # xb = open(xbfn).read()

            cfn = path(tmdir) / 'problem/p1.xml'
            assert(os.path.exists(cfn))
            os.system('ls -sFC %s' % tmdir)
            os.system('ls -sFC %s/problem' % tmdir)
            self.assertTrue(os.path.exists(cfn))
            data = open(cfn).read()
            expect = r"""[mathjax]\begin{eqnarray}
S(\rho) &amp;=&amp;  -\lambda_{1} \log \lambda_{1} -\lambda_{2} \log \lambda_{2} \\
        &amp;=&amp;  H((1+r)/2)
\end{eqnarray}[/mathjax]"""
            assert(expect in data)

if __name__ == '__main__':
    unittest.main()
